@@ -1,7 +1,9 @@
-# Copyright (c) 2017 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
+from UM.Scene.Iterator.DepthFirstIterator import DepthFirstIterator
 from UM.Scene.Scene import Scene
-from UM.Event import Event, MouseEvent, ToolEvent, ViewEvent
+from UM.Event import Event, KeyEvent, MouseEvent, ToolEvent, ViewEvent
+from UM.Scene.SceneNode import SceneNode
 from UM.Signal import Signal, signalemitter
 from UM.Logger import Logger
 from UM.PluginRegistry import PluginRegistry
@@ -10,59 +12,53 @@ from UM.PluginRegistry import PluginRegistry
 from UM.View.View import View
 from UM.Stage import Stage
 from UM.InputDevice import InputDevice
-from typing import Optional, Dict
+from typing import cast, Optional, Dict, Union
 from UM.Math.Vector import Vector
-
 MYPY = False
 if MYPY:
     from UM.Application import Application
     from UM.Tool import Tool
 
-##      Glue class that holds the scene, (active) view(s), (active) tool(s) and possible user inputs.
+
+##  Glue class that holds the scene, (active) view(s), (active) tool(s) and possible user inputs.
 #
-#       The different types of views / tools / inputs are defined by plugins.
-#       \sa View
-#       \sa Tool
-#       \sa Scene
+#   The different types of views / tools / inputs are defined by plugins.
+#   \sa View
+#   \sa Tool
+#   \sa Scene
 @signalemitter
 class Controller:
-    def __init__(self, application: "Application"):
+    def __init__(self, application: "Application") -> None:
         super().__init__()  # Call super to make multiple inheritance work.
 
         self._scene = Scene()
         self._application = application
-        self._is_model_rendering_enabled = True
 
         self._active_view = None  # type: Optional[View]
         self._views = {}  # type: Dict[str, View]
 
         self._active_tool = None  # type: Optional[Tool]
+        self._fallback_tool = "TranslateTool"  # type: str
         self._tool_operation_active = False
         self._tools = {}  # type: Dict[str, Tool]
-        self._camera_tool = None
-        self._selection_tool = None
-        self._tools_enabled = True
+        self._camera_tool = None #type: Optional[Tool]
+        self._selection_tool = None #type: Optional[Tool]
+        self._tools_enabled = True #type: bool
 
-        self._active_stage = None
-        self._stages = {}
+        self._active_stage = None #type: Optional[Stage]
+        self._stages = {} #type: Dict[str, Stage]
 
-        self._input_devices = {}
+        self._input_devices = {} #type: Dict[str, InputDevice]
 
         PluginRegistry.addType("stage", self.addStage)
         PluginRegistry.addType("view", self.addView)
         PluginRegistry.addType("tool", self.addTool)
         PluginRegistry.addType("input_device", self.addInputDevice)
 
-    ##  Get the application.
-    #   \returns Application \type {Application}
-    def getApplication(self):
-        return self._application
-
     ##  Add a view by name if it"s not already added.
-    #   \param name \type{string} Unique identifier of view (usually the plugin name)
     #   \param view \type{View} The view to be added
-    def addView(self, view: View):
-        name = view.getPluginId()
+    def addView(self, view: View) -> None:
+        name = view.getId()
         if name not in self._views:
             self._views[name] = view
             view.setRenderer(self._application.getRenderer())
@@ -71,7 +67,6 @@ class Controller:
             Logger.log("w", "%s was already added to view list. Unable to add it again.", name)
 
     ##  Request view by name. Returns None if no view is found.
-    #   \param name \type{string} Unique identifier of view (usually the plugin name)
     #   \return View \type{View} if name was found, none otherwise.
     def getView(self, name: str) -> Optional[View]:
         try:
@@ -92,7 +87,7 @@ class Controller:
 
     ##  Set the currently active view.
     #   \param name \type{string} The name of the view to set as active
-    def setActiveView(self, name: str):
+    def setActiveView(self, name: str) -> None:
         Logger.log("d", "Setting active view to %s", name)
         try:
             if self._active_view:
@@ -109,15 +104,6 @@ class Controller:
         except Exception as e:
             Logger.logException("e", "An exception occurred while switching views: %s", str(e))
 
-    def enableModelRendering(self):
-        self._is_model_rendering_enabled = True
-
-    def disableModelRendering(self):
-        self._is_model_rendering_enabled = False
-
-    def isModelRenderingEnabled(self):
-        return self._is_model_rendering_enabled
-
     ##  Emitted when the list of views changes.
     viewsChanged = Signal()
 
@@ -127,8 +113,8 @@ class Controller:
     ##  Add a stage by name if it's not already added.
     #   \param name \type{string} Unique identifier of stage (usually the plugin name)
     #   \param stage \type{Stage} The stage to be added
-    def addStage(self, stage: Stage):
-        name = stage.getPluginId()
+    def addStage(self, stage: Stage) -> None:
+        name = stage.getId()
         if name not in self._stages:
             self._stages[name] = stage
             self.stagesChanged.emit()
@@ -155,11 +141,19 @@ class Controller:
 
     ##  Set the currently active stage.
     #   \param name \type{string} The name of the stage to set as active
-    def setActiveStage(self, name: str):
-        Logger.log("d", "Setting active stage to %s", name)
+    def setActiveStage(self, name: str) -> None:
         try:
-            self._active_stage = self._stages[name]
-            self.activeStageChanged.emit()
+            # Don't actually change the stage if it is the current selected one.
+            if self._active_stage != self._stages[name]:
+                previous_stage = self._active_stage
+                Logger.log("d", "Setting active stage to %s", name)
+                self._active_stage = self._stages[name]
+
+                # If there is no error switching stages, then finish first the previous stage (if it exists) and start the new stage
+                if previous_stage is not None:
+                    previous_stage.onStageDeselected()
+                self._active_stage.onStageSelected()
+                self.activeStageChanged.emit()
         except KeyError:
             Logger.log("e", "No stage named %s found", name)
         except Exception as e:
@@ -173,8 +167,8 @@ class Controller:
 
     ##  Add an input device (e.g. mouse, keyboard, etc) if it's not already added.
     #   \param device The input device to be added
-    def addInputDevice(self, device: InputDevice):
-        name = device.getPluginId()
+    def addInputDevice(self, device: InputDevice) -> None:
+        name = device.getId()
         if name not in self._input_devices:
             self._input_devices[name] = device
             device.event.connect(self.event)
@@ -194,15 +188,26 @@ class Controller:
     ##  Remove an input device from the list of input devices.
     #   Does nothing if the input device is not in the list.
     #   \param name \type{string} The name of the device to remove.
-    def removeInputDevice(self, name: str):
+    def removeInputDevice(self, name: str) -> None:
         if name in self._input_devices:
             self._input_devices[name].event.disconnect(self.event)
             del self._input_devices[name]
 
-    ##  Request tool by name. Returns None if no view is found.
+    ##  Request the current fallbacl tool.
+    #   \return Id of the fallback tool
+    def getFallbackTool(self) -> str:
+        return self._fallback_tool
+
+    ##  Set the current active tool. The tool must be set by name.
+    #   \param tool The tools name which shall be used as fallback
+    def setFallbackTool(self, tool: str) -> None:
+        if self._fallback_tool is not tool:
+            self._fallback_tool = tool
+
+    ##  Request tool by name. Returns None if no tool is found.
     #   \param name \type{string} Unique identifier of tool (usually the plugin name)
-    #   \return tool \type{Tool} if name was found, none otherwise.
-    def getTool(self, name: str):
+    #   \return tool \type{Tool} if name was found, None otherwise.
+    def getTool(self, name: str) -> Optional["Tool"]:
         try:
             return self._tools[name]
         except KeyError:  # No such tool
@@ -211,13 +216,13 @@ class Controller:
 
     ##  Get all tools
     #   \return tools \type{dict}
-    def getAllTools(self):
+    def getAllTools(self) -> Dict[str, "Tool"]:
         return self._tools
 
     ##  Add a Tool (transform object, translate object) if its not already added.
     #   \param tool \type{Tool} Tool to be added
-    def addTool(self, tool):
-        name = tool.getPluginId()
+    def addTool(self, tool: "Tool") -> None:
+        name = tool.getId()
         if name not in self._tools:
             self._tools[name] = tool
             tool.operationStarted.connect(self._onToolOperationStarted)
@@ -226,12 +231,12 @@ class Controller:
         else:
             Logger.log("w", "%s was already added to tool list. Unable to add it again.", name)
 
-    def _onToolOperationStarted(self, tool):
+    def _onToolOperationStarted(self, tool: "Tool") -> None:
         if not self._tool_operation_active:
             self._tool_operation_active = True
             self.toolOperationStarted.emit(tool)
 
-    def _onToolOperationStopped(self, tool):
+    def _onToolOperationStopped(self, tool: "Tool") -> None:
         if self._tool_operation_active:
             self._tool_operation_active = False
             self.toolOperationStopped.emit(tool)
@@ -242,20 +247,20 @@ class Controller:
         return self._tool_operation_active
 
     ##  Request active tool. Returns None if there is no active tool
-    #   \return Tool \type{Tool} if an tool is active, None otherwise.
-    def getActiveTool(self):
+    #   \return Tool if a tool is active, None otherwise.
+    def getActiveTool(self) -> Optional["Tool"]:
         return self._active_tool
 
     ##  Set the current active tool.
     #   The tool can be set by name of the tool or directly passing the tool object.
-    #   \param tool \type{Tool} or \type{string}
-    def setActiveTool(self, tool):
+    #   \param tool A tool object or the name of a tool.
+    def setActiveTool(self, tool: Optional[Union["Tool", str]]):
         from UM.Tool import Tool
         if self._active_tool:
             self._active_tool.event(ToolEvent(ToolEvent.ToolDeactivateEvent))
 
         if isinstance(tool, Tool) or tool is None:
-            new_tool = tool
+            new_tool = cast(Optional[Tool], tool)
         else:
             new_tool = self.getTool(tool)
 
@@ -269,14 +274,16 @@ class Controller:
 
         from UM.Scene.Selection import Selection  # Imported here to prevent a circular dependency.
         if not self._active_tool and Selection.getCount() > 0:  # If something is selected, a tool must always be active.
-            if "TranslateTool" in self._tools:
-                self._active_tool = self._tools["TranslateTool"]  # Then default to the translation tool.
+            if self._fallback_tool in self._tools:
+                self._active_tool = self._tools[self._fallback_tool]  # Then default to the translation tool.
                 self._active_tool.event(ToolEvent(ToolEvent.ToolActivateEvent))
                 tool_changed = True
             else:
                 Logger.log("w", "Controller does not have an active tool and could not default to Translate tool.")
 
         if tool_changed:
+            Selection.setFaceSelectMode(False)
+            Selection.clearFace()
             self.activeToolChanged.emit()
 
     ##  Emitted when the list of tools changes.
@@ -307,33 +314,34 @@ class Controller:
 
     ##  Process an event
     #   \param event \type{Event} event to be handle.
-    #   The event is first passed to the camera tool, then active tool and finally selection tool.
+    #   The event is first passed to the selection tool, then the active tool and finally the camera tool.
     #   If none of these events handle it (when they return something that does not evaluate to true)
     #   a context menu signal is emitted.
     def event(self, event: Event):
-        # First, try to perform camera control
+        if self._selection_tool and self._selection_tool.event(event):
+            return
+
+        if self._active_tool and self._active_tool.event(event):
+            return
+
         if self._camera_tool and self._camera_tool.event(event):
             return
 
         if self._tools and event.type == Event.KeyPressEvent:
+            event = cast(KeyEvent, event)
             from UM.Scene.Selection import Selection  # Imported here to prevent a circular dependency.
             if Selection.hasSelection():
                 for key, tool in self._tools.items():
                     if tool.getShortcutKey() is not None and event.key == tool.getShortcutKey():
                         self.setActiveTool(tool)
 
-        if self._selection_tool and self._selection_tool.event(event):
-            return
-
-        # If we are not doing camera control, pass the event to the active tool.
-        if self._active_tool and self._active_tool.event(event):
-            return
-
         if self._active_view:
             self._active_view.event(event)
 
-        if event.type == Event.MouseReleaseEvent and MouseEvent.RightButton in event.buttons:
-            self.contextMenuRequested.emit(event.x, event.y)
+        if event.type == Event.MouseReleaseEvent:
+            event = cast(MouseEvent, event)
+            if MouseEvent.RightButton in event.buttons:
+                self.contextMenuRequested.emit(event.x, event.y)
 
     contextMenuRequested = Signal()
 
@@ -343,16 +351,16 @@ class Controller:
     #   \param tool \type{Tool} or \type{string}
     #   \sa setSelectionTool
     #   \sa setActiveTool
-    def setCameraTool(self, tool):
+    def setCameraTool(self, tool: Union["Tool", str]):
         from UM.Tool import Tool
         if isinstance(tool, Tool) or tool is None:
-            self._camera_tool = tool
+            self._camera_tool = cast(Optional[Tool], tool)
         else:
             self._camera_tool = self.getTool(tool)
 
     ##  Get the camera tool (if any)
     #   \returns camera tool (or none)
-    def getCameraTool(self):
+    def getCameraTool(self) -> Optional["Tool"]:
         return self._camera_tool
 
     ##  Set the tool used for performing selections.
@@ -361,42 +369,100 @@ class Controller:
     #   \param tool \type{Tool} or \type{string}
     #   \sa setCameraTool
     #   \sa setActiveTool
-    def setSelectionTool(self, tool):
+    def setSelectionTool(self, tool: Union[str, "Tool"]):
         from UM.Tool import Tool
         if isinstance(tool, Tool) or tool is None:
-            self._selection_tool = tool
+            self._selection_tool = cast(Optional[Tool], tool)
         else:
             self._selection_tool = self.getTool(tool)
 
-    def getToolsEnabled(self):
+    def getToolsEnabled(self) -> bool:
         return self._tools_enabled
 
-    def setToolsEnabled(self, enabled):
+    def setToolsEnabled(self, enabled: bool) -> None:
         self._tools_enabled = enabled
 
+    def deleteAllNodesWithMeshData(self, only_selectable:bool = True) -> None:
+        Logger.log("i", "Clearing scene")
+        if not self.getToolsEnabled():
+            return
+
+        nodes = []
+        for node in DepthFirstIterator(self.getScene().getRoot()):
+            if not node.isEnabled():
+                continue
+            if not node.getMeshData() and not node.callDecoration("isGroup"):
+                continue  # Node that doesnt have a mesh and is not a group.
+            if only_selectable and not node.isSelectable():
+                continue  # Only remove nodes that are selectable.
+            if node.getParent() and cast(SceneNode, node.getParent()).callDecoration("isGroup"):
+                continue  # Grouped nodes don't need resetting as their parent (the group) is resetted)
+            nodes.append(node)
+        if nodes:
+            from UM.Operations.GroupedOperation import GroupedOperation
+            op = GroupedOperation()
+
+            for node in nodes:
+                from UM.Operations.RemoveSceneNodeOperation import RemoveSceneNodeOperation
+                op.addOperation(RemoveSceneNodeOperation(node))
+
+                # Reset the print information
+                self.getScene().sceneChanged.emit(node)
+
+            op.push()
+            from UM.Scene.Selection import Selection
+            Selection.clear()
+
     # Rotate camera view according defined angle
-    def rotateView(self, coordinate = "x", angle = 0):
+    def setCameraRotation(self, coordinate: str = "x", angle: int = 0) -> None:
         camera = self._scene.getActiveCamera()
-        self._camera_tool.setOrigin(Vector(0, 100, 0))
+        if not camera:
+            return
+        camera.setZoomFactor(camera.getDefaultZoomFactor())
+        self._camera_tool.setOrigin(Vector(0, 100, 0))  # type: ignore
+        self.setCameraOrigin(coordinate)
         if coordinate == "home":
-            camera.setPosition(Vector(0, 0, 700))
-            camera.setPerspective(True)
-            camera.lookAt(Vector(0, 100, 100))
-            self._camera_tool.rotateCam(0, 0)
+            camera.setPosition(Vector(0, 100, 700))
+            self._camera_tool.rotateCamera(0, 0)  # type: ignore
         elif coordinate == "3d":
             camera.setPosition(Vector(-750, 600, 700))
-            camera.setPerspective(True)
-            camera.lookAt(Vector(0, 100, 100))
-            self._camera_tool.rotateCam(0, 0)
-
+            self._camera_tool.rotateCamera(0, 0)  # type: ignore
         else:
             # for comparison is == used, because might not store them at the same location
             # https://stackoverflow.com/questions/1504717/why-does-comparing-strings-in-python-using-either-or-is-sometimes-produce
-            camera.setPosition(Vector(0, 0, 700))
-            camera.setPerspective(True)
-            camera.lookAt(Vector(0, 100, 0))
 
             if coordinate == "x":
-                self._camera_tool.rotateCam(angle, 0)
+                camera.setPosition(Vector(0, 100, 700))
+                self._camera_tool.rotateCamera(angle, 0)  # type: ignore
             elif coordinate == "y":
-                self._camera_tool.rotateCam(0, angle)
\ No newline at end of file
+                if angle == 90:
+                    # Prepare the camera for top view, so no rotation has to be applied after setting the top view.
+                    camera.setPosition(Vector(0, 100, 100))
+                    self._camera_tool.rotateCamera(90, 0)  # type: ignore
+                    # Actually set the top view.
+                    camera.setPosition(Vector(0, 800, 1))
+                    self.setCameraOrigin("z")
+                    camera.lookAt(Vector(0, 100, 1))
+                    self._camera_tool.rotateCamera(0, 0)  # type: ignore
+                else:
+                    camera.setPosition(Vector(0, 100, 700))
+                    self._camera_tool.rotateCamera(0, angle)  # type: ignore
+
+    ##  Changes the origin of the camera, i.e. where it looks at.
+    #   \param coordinate One of the following options:
+    #    - "home": The centre of the build plate.
+    #    - "3d": The centre of the build volume.
+    #    - "x", "y" and "z": Also the centre of the build plate. These are just
+    #      aliases for the setCameraRotation function.
+    def setCameraOrigin(self, coordinate: str = "home"):
+        camera = self._scene.getActiveCamera()
+        if not camera:
+            return
+        coordinates = {
+            "home": Vector(0, 100, 0),
+            "3d": Vector(0, 100, 100),
+            "x": Vector(0, 100, 0),
+            "y": Vector(0, 100, 0),
+            "z": Vector(0, 100, 1)
+        }
+        camera.lookAt(coordinates[coordinate])
\ No newline at end of file
