@@ -3,7 +3,6 @@
 
 from UM.Message import Message
 from UM.Math.Vector import Vector
-from UM.Preferences import Preferences
 from UM.Logger import Logger
 
 from UM.FileHandler.ReadFileJob import ReadFileJob
@@ -13,6 +12,7 @@ import math
 from UM.i18n import i18nCatalog
 i18n_catalog = i18nCatalog("uranium")
 
+
 ##  A Job subclass that performs mesh loading.
 #
 #   The result of this Job is a MeshData object.
@@ -36,19 +36,22 @@ class ReadMeshJob(ReadFileJob):
                 node._resetAABB()
                 build_bounds = node.getBoundingBox()
 
-                if Preferences.getInstance().getValue("mesh/scale_to_fit") == True or Preferences.getInstance().getValue("mesh/scale_tiny_meshes") == True:
+                if build_bounds is None or max_bounds is None:
+                    continue
+
+                if self._application.getInstance().getPreferences().getValue("mesh/scale_to_fit") == True or self._application.getInstance().getPreferences().getValue("mesh/scale_tiny_meshes") == True:
                     scale_factor_width = max_bounds.width / build_bounds.width
                     scale_factor_height = max_bounds.height / build_bounds.height
                     scale_factor_depth = max_bounds.depth / build_bounds.depth
                     scale_factor = min(scale_factor_width, scale_factor_depth, scale_factor_height)
-                    if Preferences.getInstance().getValue("mesh/scale_to_fit") == True and (scale_factor_width < 1 or scale_factor_height < 1 or scale_factor_depth < 1): # Use scale factor to scale large object down
+                    if self._application.getInstance().getPreferences().getValue("mesh/scale_to_fit") == True and (scale_factor_width < 1 or scale_factor_height < 1 or scale_factor_depth < 1): # Use scale factor to scale large object down
                         # Ignore scaling on models which are less than 1.25 times bigger than the build volume
                         ignore_factor = 1.25
                         if 1 / scale_factor < ignore_factor:
                             Logger.log("i", "Ignoring auto-scaling, because %.3d < %.3d" % (1 / scale_factor, ignore_factor))
                             scale_factor = 1
                         pass
-                    elif Preferences.getInstance().getValue("mesh/scale_tiny_meshes") == True and (scale_factor_width > 100 and scale_factor_height > 100 and scale_factor_depth > 100):
+                    elif self._application.getInstance().getPreferences().getValue("mesh/scale_tiny_meshes") == True and (scale_factor_width > 100 and scale_factor_height > 100 and scale_factor_depth > 100):
                         # Round scale factor to lower factor of 10 to scale tiny object up (eg convert m to mm units)
                         try:
                             scale_factor = math.pow(10, math.floor(math.log(scale_factor) / math.log(10)))
