@@ -1,12 +1,14 @@
-# Copyright (c) 2015 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import configparser
+from typing import Any, Dict, IO, Optional, Tuple, Union
 
-from UM.Signal import Signal, signalemitter
+from UM.Decorators import deprecated
 from UM.Logger import Logger
 from UM.MimeTypeDatabase import MimeTypeDatabase, MimeType #To register the MIME type of the preference file.
 from UM.SaveFile import SaveFile
+from UM.Signal import Signal, signalemitter
 
 MimeTypeDatabase.addMimeType(
     MimeType(
@@ -17,23 +19,27 @@ MimeTypeDatabase.addMimeType(
     )
 )
 
+
 ##      Preferences are application based settings that are saved for future use.
 #       Typical preferences would be window size, standard machine, etc.
+#       The application preferences can be gotten from the getPreferences() function in Application
 @signalemitter
 class Preferences:
     Version = 6
 
-    def __init__(self):
+    def __init__(self) -> None:
         super().__init__()
 
-        self._file = None
-        self._parser = None
-        self._preferences = {}
+        self._parser = None  # type: Optional[configparser.ConfigParser]
+        self._preferences = {}  # type: Dict[str, Dict[str, _Preference]]
 
-    def addPreference(self, key, default_value):
+    ##  Add a new preference to the list. If the preference was already added, it's default is set to whatever is provided
+    def addPreference(self, key: str, default_value: Any) -> None:
+        if key.count("/") != 1:
+            raise Exception("Preferences must be in the [CATEGORY]/[KEY] format")
         preference = self._findPreference(key)
         if preference:
-            preference.setDefault(default_value)
+            self.setDefault(key, default_value)
             return
 
         group, key = self._splitKey(key)
@@ -59,25 +65,25 @@ class Preferences:
     #
     #   \param key The key of the preference to set the default of.
     #   \param default_value The new default value of the preference.
-    def setDefault(self, key, default_value):
+    def setDefault(self, key: str, default_value: Any) -> None:
         preference = self._findPreference(key)
-        if not preference: #Key not found.
+        if not preference:  # Key not found.
             Logger.log("w", "Tried to set the default value of non-existing setting %s.", key)
             return
         if preference.getValue() == preference.getDefault():
             self.setValue(key, default_value)
         preference.setDefault(default_value)
 
-    def setValue(self, key, value):
+    def setValue(self, key: str, value: Any) -> None:
         preference = self._findPreference(key)
-
         if preference:
-            preference.setValue(value)
-            self.preferenceChanged.emit(key)
+            if preference.getValue() != value:
+                preference.setValue(value)
+                self.preferenceChanged.emit(key)
         else:
             Logger.log("w", "Tried to set the value of non-existing setting %s.", key)
 
-    def getValue(self, key):
+    def getValue(self, key: str) -> Any:
         preference = self._findPreference(key)
 
         if preference:
@@ -91,22 +97,25 @@ class Preferences:
         Logger.log("w", "Tried to get the value of non-existing setting %s.", key)
         return None
 
-    def resetPreference(self, key):
+    def resetPreference(self, key: str) -> None:
         preference = self._findPreference(key)
 
         if preference:
-            preference.setValue(preference.getDefault())
-            self.preferenceChanged.emit(key)
+            if preference.getValue() != preference.getDefault():
+                preference.setValue(preference.getDefault())
+                self.preferenceChanged.emit(key)
+        else:
+            Logger.log("w", "Tried to reset unknown setting %s", key)
 
-    def readFromFile(self, file):
+    def readFromFile(self, file: Union[str, IO[str]]) -> None:
         self._loadFile(file)
-
-        if not self._parser:
-            return
-
         self.__initializeSettings()
 
-    def __initializeSettings(self):
+    def __initializeSettings(self) -> None:
+        if self._parser is None:
+            Logger.log("w", "Read the preferences file before initializing settings!")
+            return
+
         for group, group_entries in self._parser.items():
             if group == "DEFAULT":
                 continue
@@ -121,7 +130,7 @@ class Preferences:
                 self._preferences[group][key].setValue(value)
                 self.preferenceChanged.emit("{0}/{1}".format(group, key))
 
-    def writeToFile(self, file):
+    def writeToFile(self, file: Union[str, IO[str]]) -> None:
         parser = configparser.ConfigParser(interpolation = None) #pylint: disable=bad-whitespace
         for group, group_entries in self._preferences.items():
             parser[group] = {}
@@ -133,7 +142,7 @@ class Preferences:
 
         try:
             if hasattr(file, "read"):  # If it already is a stream like object, write right away
-                parser.write(file)
+                parser.write(file) #type: ignore #Can't convince MyPy that it really is an IO object now.
             else:
                 with SaveFile(file, "wt") as save_file:
                     parser.write(save_file)
@@ -142,14 +151,7 @@ class Preferences:
 
     preferenceChanged = Signal()
 
-    @classmethod
-    def getInstance(cls) -> "Preferences":
-        if not cls._instance:
-            cls._instance = Preferences()
-
-        return cls._instance
-
-    def _splitKey(self, key):
+    def _splitKey(self, key: str) -> Tuple[str, str]:
         group = "general"
         key = key
 
@@ -158,9 +160,9 @@ class Preferences:
             group = parts[0]
             key = parts[1]
 
-        return (group, key)
+        return group, key
 
-    def _findPreference(self, key):
+    def _findPreference(self, key: str) -> Optional[Any]:
         group, key = self._splitKey(key)
 
         if group in self._preferences:
@@ -169,9 +171,7 @@ class Preferences:
 
         return None
 
-    def _loadFile(self, file):
-        if self._file and self._file == file:
-            return self._parser
+    def _loadFile(self, file: Union[str, IO[str]]) -> None:
         try:
             self._parser = configparser.ConfigParser(interpolation = None) #pylint: disable=bad-whitespace
             if hasattr(file, "read"):
@@ -184,20 +184,23 @@ class Preferences:
                 self._parser = None
                 return
         except Exception:
-            Logger.logException("e", "An exception occured while trying to read preferences file")
+            Logger.logException("e", "An exception occurred while trying to read preferences file")
             self._parser = None
             return
 
         del self._parser["general"]["version"]
 
-    _instance = None  # type: Preferences
-
     ##  Extract data from string and store it in the Configuration parser.
-    def deserialize(self, serialized: str) -> str:
+    def deserialize(self, serialized: str) -> None:
         updated_preferences = self.__updateSerialized(serialized)
-        self._parser = configparser.ConfigParser(interpolation=None)
-        self._parser.read_string(updated_preferences)
-        has_version = "version" in self._parser["general"]
+        self._parser = configparser.ConfigParser(interpolation = None)
+        try:
+            self._parser.read_string(updated_preferences)
+        except configparser.MissingSectionHeaderError:
+            Logger.log("w", "Could not deserialize preferences from loaded project")
+            self._parser = None
+            return
+        has_version = "general" in self._parser and "version" in self._parser["general"]
 
         if has_version:
             if self._parser["general"]["version"] != str(Preferences.Version):
@@ -213,40 +216,35 @@ class Preferences:
     def __updateSerialized(self, serialized: str) -> str:
         configuration_type = "preferences"
 
-        version = None
         try:
-            import UM.VersionUpgradeManager
-            version = UM.VersionUpgradeManager.VersionUpgradeManager.getInstance().getFileVersion(configuration_type,
-                                                                                                  serialized)
+            from UM.VersionUpgradeManager import VersionUpgradeManager
+            version = VersionUpgradeManager.getInstance().getFileVersion(configuration_type, serialized)
             if version is not None:
-                from UM.VersionUpgradeManager import VersionUpgradeManager
-                result = VersionUpgradeManager.getInstance().updateFilesData(configuration_type, version,
-                                                                             [serialized], [""])
+                result = VersionUpgradeManager.getInstance().updateFilesData(configuration_type, version, [serialized], [""])
                 if result is not None:
                     serialized = result.files_data[0]
-            return serialized
+        except:
+            Logger.logException("d", "An exception occurred while trying to update the preferences.")
+        return serialized
 
-        except Exception as e:
-            Logger.log("d", "Could not get version from serialized: %s", e)
-            pass
 
 class _Preference:
-    def __init__(self, name, default = None, value = None): #pylint: disable=bad-whitespace
+    def __init__(self, name: str, default: Any = None, value: Any = None) -> None:
         self._name = name
         self._default = default
         self._value = default if value is None else value
 
-    def getName(self):
+    def getName(self) -> str:
         return self._name
 
-    def getValue(self):
+    def getValue(self) -> Any:
         return self._value
 
-    def getDefault(self):
+    def getDefault(self) -> Any:
         return self._default
 
-    def setDefault(self, default):
+    def setDefault(self, default: Any) -> None:
         self._default = default
 
-    def setValue(self, value):
-        self._value = value
\ No newline at end of file
+    def setValue(self, value: Any) -> None:
+        self._value = value
