@@ -1,13 +1,15 @@
 # Copyright (c) 2015 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
-from PyQt5.QtCore import QObject, QCoreApplication, pyqtSlot, QUrl, pyqtSignal, pyqtProperty
+from PyQt5.QtCore import QObject, pyqtSlot, pyqtSignal, pyqtProperty
 
 from UM.Application import Application
+from UM.Decorators import deprecated
 from UM.Scene.Selection import Selection
 from UM.Operations.RemoveSceneNodeOperation import RemoveSceneNodeOperation
 from UM.Operations.GroupedOperation import GroupedOperation
 
+
 class ControllerProxy(QObject):
     def __init__(self, parent = None):
         super().__init__(parent)
@@ -20,9 +22,11 @@ class ControllerProxy(QObject):
         self._controller.toolOperationStarted.connect(self._onToolOperationStarted)
         self._controller.toolOperationStopped.connect(self._onToolOperationStopped)
         self._controller.activeStageChanged.connect(self._onActiveStageChanged)
+        self._controller.activeViewChanged.connect(self._onActiveViewChanged)
 
     toolsEnabledChanged = pyqtSignal()
     activeStageChanged = pyqtSignal()
+    activeViewChanged = pyqtSignal()
 
     @pyqtProperty(bool, notify = toolsEnabledChanged)
     def toolsEnabled(self):
@@ -36,6 +40,10 @@ class ControllerProxy(QObject):
     def setActiveView(self, view):
         self._controller.setActiveView(view)
 
+    @pyqtProperty(QObject, notify = activeViewChanged)
+    def activeView(self):
+        return self._controller.getActiveView()
+
     @pyqtSlot(str)
     def setActiveStage(self, stage):
         self._controller.setActiveStage(stage)
@@ -55,21 +63,17 @@ class ControllerProxy(QObject):
         op.push()
         Selection.clear()
 
-    @pyqtSlot()
-    def enableModelRendering(self):
-        self._controller.enableModelRendering()
-
-    @pyqtSlot()
-    def disableModelRendering(self):
-        self._controller.disableModelRendering()
-
     @pyqtSlot(str, int)
-    def rotateView(self,coordinate, angle):
-        self._controller.rotateView(coordinate, angle)
+    def setCameraRotation(self, coordinate: str, angle: int) -> None:
+        self._controller.setCameraRotation(coordinate, angle)
 
-    @pyqtSlot()
-    def homeView(self, angle):
-        self._controller.homeView()
+    ##  Changes the position of the origin of the camera.
+    #   \param coordinate The new origin of the camera. Use either:
+    #     "home": The centre of the build plate.
+    #     "3d": The centre of the build volume.
+    @pyqtSlot(str)
+    def setCameraOrigin(self, coordinate: str) -> None:
+        self._controller.setCameraOrigin(coordinate)
 
     contextMenuRequested = pyqtSignal("quint64", arguments=["objectId"])
 
@@ -96,3 +100,6 @@ class ControllerProxy(QObject):
 
     def _onActiveStageChanged(self):
         self.activeStageChanged.emit()
+
+    def _onActiveViewChanged(self):
+        self.activeViewChanged.emit()
