@@ -17,6 +17,10 @@ class VisibleMessagesModel(ListModel):
     DismissableRole = Qt.UserRole + 8
     TileRole = Qt.UserRole + 9
     StyleRole = Qt.UserRole + 10
+    ImageSourceRole = Qt.UserRole + 11
+    ImageCaptionRole = Qt.UserRole + 12
+    OptionTextRole = Qt.UserRole + 13
+    OptionStateRole = Qt.UserRole + 14
 
     def __init__(self, parent=None):
         super().__init__(parent)
@@ -29,6 +33,10 @@ class VisibleMessagesModel(ListModel):
         self.addRoleName(self.ActionsRole, "actions")
         self.addRoleName(self.DismissableRole, "dismissable")
         self.addRoleName(self.TileRole, "title")
+        self.addRoleName(self.ImageSourceRole, "image_source")
+        self.addRoleName(self.ImageCaptionRole, "image_caption")
+        self.addRoleName(self.OptionTextRole, "option_text")
+        self.addRoleName(self.OptionStateRole, "option_state")
         self._populateMessageList()
 
     def _populateMessageList(self):
@@ -43,8 +51,11 @@ class VisibleMessagesModel(ListModel):
             "id": str(id(message)),
             "actions": self.createActionsModel(message.getActions()),
             "dismissable": message.isDismissable(),
-            "title": message.getTitle()
-
+            "title": message.getTitle(),
+            "image_source": message.getImageSource(),
+            "image_caption": message.getImageCaption(),
+            "option_text": message.getOptionText(),
+            "option_state": message.getOptionState()
         })
         message.progressChanged.connect(self._onMessageProgress)
 
@@ -64,6 +75,13 @@ class VisibleMessagesModel(ListModel):
     def hideMessage(self, message_id):
         Application.getInstance().hideMessageById(message_id)
 
+    @pyqtSlot(str, bool)
+    def optionToggled(self, message_id, value):
+        for message in Application.getInstance().getVisibleMessages():
+            if str(id(message)) == message_id:
+                message.optionToggled.emit(value)
+                break
+
     @pyqtSlot(str, str)
     def actionTriggered(self, message_id, action_id):
         for message in Application.getInstance().getVisibleMessages():
