@@ -1,13 +1,18 @@
 # Copyright (c) 2018 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
-from UM.Scene.SceneNode import SceneNode
 
-from typing import List, Iterable
+from typing import List, TYPE_CHECKING
+
+import typing
+
+if TYPE_CHECKING:
+    from UM.Scene.SceneNode import SceneNode
+
 
 ##    Abstract iterator class.
 class Iterator:
-    def __init__(self, scene_node: SceneNode):
-        super().__init__() # Call super to make multiple inheritence work.
+    def __init__(self, scene_node: "SceneNode") -> None:
+        super().__init__()  # Call super to make multiple inheritance work.
         self._scene_node = scene_node
         self._node_stack = []  # type: List[SceneNode]
         self._fillStack()
@@ -16,5 +21,5 @@ class Iterator:
     def _fillStack(self) -> None:
         raise NotImplementedError("Iterator is not correctly implemented. Requires a _fill_stack implementation.")
     
-    def __iter__(self) -> Iterable[SceneNode]:
-        return iter(self._node_stack)
\ No newline at end of file
+    def __iter__(self) -> typing.Iterator["SceneNode"]:
+        return iter(self._node_stack)
