@@ -1,21 +1,25 @@
-# Copyright (c) 2017 Ultimaker B.V.
+# Copyright (c) 2019 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import configparser
 import io
-from typing import Any, cast, Dict, List, Optional, Set
+from typing import Any, cast, Dict, List, Optional, Set, Tuple
 
 from PyQt5.QtCore import QObject, pyqtProperty, pyqtSignal
+
+from UM.Logger import Logger
+from PyQt5.QtQml import QQmlEngine
 import UM.FlameProfiler
 
-from UM.Settings.SettingDefinition import SettingDefinition
+from UM.ConfigurationErrorMessage import ConfigurationErrorMessage
 from UM.Signal import Signal, signalemitter
 from UM.PluginObject import PluginObject
-from UM.Logger import Logger
 from UM.MimeTypeDatabase import MimeTypeDatabase, MimeType
+from UM.Settings.ContainerFormatError import ContainerFormatError
 from UM.Settings.DefinitionContainer import DefinitionContainer #For getting all definitions in this stack.
 from UM.Settings.Interfaces import ContainerInterface, ContainerRegistryInterface
 from UM.Settings.PropertyEvaluationContext import PropertyEvaluationContext
+from UM.Settings.SettingDefinition import SettingDefinition
 from UM.Settings.SettingFunction import SettingFunction
 from UM.Settings.Validator import ValidatorState
 
@@ -27,6 +31,7 @@ class IncorrectVersionError(Exception):
 class InvalidContainerStackError(Exception):
     pass
 
+
 MimeTypeDatabase.addMimeType(
     MimeType(
         name = "application/x-uranium-containerstack",
@@ -47,49 +52,48 @@ MimeTypeDatabase.addMimeType(
 ##  A stack of setting containers to handle setting value retrieval.
 @signalemitter
 class ContainerStack(QObject, ContainerInterface, PluginObject):
-    Version = 4 # type: int
+    Version = 4  # type: int
 
     ##  Constructor
     #
-    #   \param stack_id \type{string} A unique, machine readable/writable ID.
-    def __init__(self, stack_id: str, parent = None, *args, **kwargs):
-        # Note that we explicitly pass None as QObject parent here. This is to be able
-        # to support pickling.
-        super().__init__(parent = parent, *args, **kwargs)
+    #   \param stack_id A unique, machine readable/writable ID.
+    def __init__(self, stack_id: str) -> None:
+        super().__init__()
+        QQmlEngine.setObjectOwnership(self, QQmlEngine.CppOwnership)
 
         self._metadata = {
             "id": stack_id,
             "name": stack_id,
             "version": self.Version,
             "container_type": ContainerStack
-        }
+        } #type: Dict[str, Any]
         self._containers = []  # type: List[ContainerInterface]
         self._next_stack = None  # type: Optional[ContainerStack]
         self._read_only = False  # type: bool
-        self._dirty = True  # type: bool
+        self._dirty = False  # type: bool
         self._path = ""  # type: str
-        self._postponed_emits = []  # gets filled with 2-tuples: signal, signal_argument(s)
+        self._postponed_emits = []  # type: List[Tuple[Signal, ContainerInterface]] # gets filled with 2-tuples: signal, signal_argument(s)
 
-        self._property_changes = {}
+        self._property_changes = {}  # type: Dict[str, Set[str]]
         self._emit_property_changed_queued = False  # type: bool
 
     ##  For pickle support
-    def __getnewargs__(self):
+    def __getnewargs__(self) -> Tuple[str]:
         return (self.getId(),)
 
     ##  For pickle support
-    def __getstate__(self):
+    def __getstate__(self) -> Dict[str, Any]:
         return self.__dict__
 
     ##  For pickle support
-    def __setstate__(self, state):
+    def __setstate__(self, state: Dict[str, Any]) -> None:
         self.__dict__.update(state)
 
     ##  \copydoc ContainerInterface::getId
     #
     #   Reimplemented from ContainerInterface
     def getId(self) -> str:
-        return self._metadata["id"]
+        return cast(str, self._metadata["id"])
 
     id = pyqtProperty(str, fget = getId, constant = True)
 
@@ -118,7 +122,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     def isReadOnly(self) -> bool:
         return self._read_only
 
-    def setReadOnly(self, read_only):
+    def setReadOnly(self, read_only: bool) -> None:
         if read_only != self._read_only:
             self._read_only = read_only
             self.readOnlyChanged.emit()
@@ -129,14 +133,25 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     ##  \copydoc ContainerInterface::getMetaData
     #
     #   Reimplemented from ContainerInterface
-    def getMetaData(self):
+    def getMetaData(self) -> Dict[str, Any]:
         return self._metadata
 
     ##  Set the complete set of metadata
-    def setMetaData(self, meta_data):
-        if meta_data != self._meta_data:
-            self._meta_data = meta_data
-            self.metaDataChanged.emit(self)
+    def setMetaData(self, meta_data: Dict[str, Any]) -> None:
+        if meta_data == self.getMetaData():
+            return #Unnecessary.
+
+        #We'll fill a temporary dictionary with all the required metadata and overwrite it with the new metadata.
+        #This way it is ensured that at least the required metadata is still there.
+        self._metadata = {
+            "id": self.getId(),
+            "name": self.getName(),
+            "version": self.getMetaData().get("version", 0),
+            "container_type": ContainerStack
+        }
+
+        self._metadata.update(meta_data)
+        self.metaDataChanged.emit(self)
 
     metaDataChanged = pyqtSignal(QObject)
     metaData = pyqtProperty("QVariantMap", fget = getMetaData, fset = setMetaData, notify = metaDataChanged)
@@ -144,7 +159,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     ##  \copydoc ContainerInterface::getMetaDataEntry
     #
     #   Reimplemented from ContainerInterface
-    def getMetaDataEntry(self, entry: str, default = None):
+    def getMetaDataEntry(self, entry: str, default: Any = None) -> Any:
         value = self._metadata.get(entry, None)
 
         if value is None:
@@ -158,23 +173,13 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
         else:
             return value
 
-    def addMetaDataEntry(self, key: str, value):
-        if key not in self._metadata:
-            self._dirty = True
+    def setMetaDataEntry(self, key: str, value: Any) -> None:
+        if key not in self._metadata or self._metadata[key] != value:
             self._metadata[key] = value
-            self.metaDataChanged.emit(self)
-        else:
-            Logger.log("w", "Meta data with key %s was already added.", key)
-
-    def setMetaDataEntry(self, key, value):
-        if key in self._metadata:
             self._dirty = True
-            self._metadata[key] = value
             self.metaDataChanged.emit(self)
-        else:
-            Logger.log("w", "Meta data with key %s was not found. Unable to change.", key)
 
-    def removeMetaDataEntry(self, key):
+    def removeMetaDataEntry(self, key: str) -> None:
         if key in self._metadata:
             del self._metadata[key]
             self.metaDataChanged.emit(self)
@@ -200,7 +205,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     #   Note that if the property value is a function, this method will return the
     #   result of evaluating that property with the current stack. If you need the
     #   actual function, use getRawProperty()
-    def getProperty(self, key: str, property_name: str, context: Optional[PropertyEvaluationContext] = None):
+    def getProperty(self, key: str, property_name: str, context: Optional[PropertyEvaluationContext] = None) -> Any:
         value = self.getRawProperty(key, property_name, context = context)
         if isinstance(value, SettingFunction):
             if context is not None:
@@ -227,8 +232,8 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     #   \return The raw property value of the property, or None if not found. Note that
     #           the value might be a SettingFunction instance.
     #
-    def getRawProperty(self, key, property_name, *, context: Optional[PropertyEvaluationContext] = None,
-                       use_next = True, skip_until_container = None):
+    def getRawProperty(self, key: str, property_name: str, *, context: Optional[PropertyEvaluationContext] = None,
+                       use_next: bool = True, skip_until_container: Optional[ContainerInterface] = None) -> Any:
         containers = self._containers
         if context is not None:
             # if context is provided, check if there is any container that needs to be skipped.
@@ -279,7 +284,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     #   Reimplemented from ContainerInterface
     #
     #   TODO: Expand documentation here, include the fact that this should _not_ include all containers
-    def serialize(self, ignored_metadata_keys: Optional[set] = None):
+    def serialize(self, ignored_metadata_keys: Optional[Set[str]] = None) -> str:
         parser = configparser.ConfigParser(interpolation = None, empty_lines_in_values = False)
 
         parser["general"] = {}
@@ -310,7 +315,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     #   the serialized data. Due to legacy problem, those data may not be available if it comes from an ancient Cura.
     @classmethod
     def _readAndValidateSerialized(cls, serialized: str) -> configparser.ConfigParser:
-        parser = configparser.ConfigParser(interpolation=None, empty_lines_in_values=False)
+        parser = configparser.ConfigParser(interpolation = None, empty_lines_in_values=False)
         parser.read_string(serialized)
 
         if "general" not in parser or any(pn not in parser["general"] for pn in ("version", "name", "id")):
@@ -324,6 +329,8 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
         try:
             parser = cls._readAndValidateSerialized(serialized)
             configuration_type = parser["metadata"]["type"]
+        except InvalidContainerStackError as icse:
+            raise icse
         except Exception as e:
             Logger.log("e", "Could not get configuration type: %s", e)
         return configuration_type
@@ -331,12 +338,15 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     @classmethod
     def getVersionFromSerialized(cls, serialized: str) -> Optional[int]:
         configuration_type = cls.getConfigurationTypeFromSerialized(serialized)
-        # get version
+        if not configuration_type:
+            Logger.log("d", "Could not get type from serialized.")
+            return None
+
+        # Get version
         version = None
         try:
-            import UM.VersionUpgradeManager
-            version = UM.VersionUpgradeManager.VersionUpgradeManager.getInstance().getFileVersion(configuration_type,
-                                                                                                  serialized)
+            from UM.VersionUpgradeManager import VersionUpgradeManager
+            version = VersionUpgradeManager.getInstance().getFileVersion(configuration_type, serialized)
         except Exception as e:
             Logger.log("d", "Could not get version from serialized: %s", e)
         return version
@@ -346,13 +356,13 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     #   Reimplemented from ContainerInterface
     #
     #   TODO: Expand documentation here, include the fact that this should _not_ include all containers
-    def deserialize(self, serialized, file_name = None) -> str:
-        # update the serialized data first
+    def deserialize(self, serialized: str, file_name: Optional[str] = None) -> str:
+        # Update the serialized data first
         serialized = super().deserialize(serialized, file_name)
         parser = self._readAndValidateSerialized(serialized)
 
-        if parser["general"].getint("version") != self.Version:
-            raise IncorrectVersionError
+        if parser.getint("general", "version") != self.Version:
+            raise IncorrectVersionError()
 
         # Clear all data before starting.
         for container in self._containers:
@@ -365,7 +375,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
             self._metadata = dict(parser["metadata"])
         self._metadata["id"] = parser["general"]["id"]
         self._metadata["name"] = parser["general"].get("name", self.getId())
-        self._metadata["version"] = self.Version #Guaranteed to be equal to what's in the container. See above.
+        self._metadata["version"] = self.Version  # Guaranteed to be equal to what's in the container. See above.
         self._metadata["container_type"] = ContainerStack
 
         if "containers" in parser:
@@ -375,12 +385,15 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
                     containers[0].propertyChanged.connect(self._collectPropertyChanges)
                     self._containers.append(containers[0])
                 else:
-                    raise Exception("When trying to deserialize %s, we received an unknown ID (%s) for container" % (self.getId(), container_id))
+                    self._containers.append(_containerRegistry.getEmptyInstanceContainer())
+                    ConfigurationErrorMessage.getInstance().addFaultyContainers(container_id, self.getId())
+                    Logger.log("e", "When trying to deserialize %s, we received an unknown container ID (%s)" % (self.getId(), container_id))
+                    raise ContainerFormatError("When trying to deserialize %s, we received an unknown container ID (%s)" % (self.getId(), container_id))
 
         elif parser.has_option("general", "containers"):
             # Backward compatibility with 2.3.1: The containers used to be saved in a single comma-separated list.
             container_string = parser["general"].get("containers", "")
-            Logger.log("d", "While deserializeing, we got the following container string: %s", container_string)
+            Logger.log("d", "While deserializing, we got the following container string: %s", container_string)
             container_id_list = container_string.split(",")
             for container_id in container_id_list:
                 if container_id != "":
@@ -389,7 +402,10 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
                         containers[0].propertyChanged.connect(self._collectPropertyChanges)
                         self._containers.append(containers[0])
                     else:
-                        raise Exception("When trying to deserialize %s, we received an unknown ID (%s) for container" % (self.getId(), container_id))
+                        self._containers.append(_containerRegistry.getEmptyInstanceContainer())
+                        ConfigurationErrorMessage.getInstance().addFaultyContainers(container_id, self.getId())
+                        Logger.log("e", "When trying to deserialize %s, we received an unknown container ID (%s)" % (self.getId(), container_id))
+                        raise ContainerFormatError("When trying to deserialize %s, we received an unknown container ID (%s)" % (self.getId(), container_id))
 
         ## TODO; Deserialize the containers.
 
@@ -408,7 +424,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     #   instead.
     @classmethod
     def deserializeMetadata(cls, serialized: str, container_id: str) -> List[Dict[str, Any]]:
-        serialized = cls._updateSerialized(serialized) #Update to most recent version.
+        serialized = cls._updateSerialized(serialized)  # Update to most recent version.
         parser = configparser.ConfigParser(interpolation = None)
         parser.read_string(serialized)
 
@@ -419,8 +435,8 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
         try:
             metadata["name"] = parser["general"]["name"]
             metadata["version"] = parser["general"]["version"]
-        except KeyError: #One of the keys or the General section itself is missing.
-            return []
+        except KeyError as e:  # One of the keys or the General section itself is missing.
+            raise InvalidContainerStackError("Missing required fields: {error_msg}".format(error_msg = str(e)))
 
         if "metadata" in parser:
             metadata.update(parser["metadata"])
@@ -434,7 +450,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     #
     #   \return A set of all setting keys in this container stack.
     def getAllKeys(self) -> Set[str]:
-        keys = set()    # type: Set[str]
+        keys = set()  # type: Set[str]
         definition_containers = [container for container in self.getContainers() if container.__class__ == DefinitionContainer] #To get all keys, get all definitions from all definition containers.
         for definition_container in cast(List[DefinitionContainer], definition_containers):
             keys |= definition_container.getAllKeys()
@@ -455,7 +471,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
 
     ##  Get a container by index.
     #
-    #   \param index \type{int} The index of the container to get.
+    #   \param index The index of the container to get.
     #
     #   \return The container at the specified index.
     #
@@ -496,11 +512,11 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     ##  \copydoc ContainerInterface::setPath
     #
     #   Reimplemented from ContainerInterface
-    def setPath(self, path: str):
+    def setPath(self, path: str) -> None:
         self._path = path
 
     ##  Get the SettingDefinition object for a specified key
-    def getSettingDefinition(self, key: str):
+    def getSettingDefinition(self, key: str) -> Optional[SettingDefinition]:
         for container in self._containers:
             if not isinstance(container, DefinitionContainer):
                 continue
@@ -516,15 +532,14 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
 
     ##  Find a container matching certain criteria.
     #
-    #   \param filter \type{dict} A dictionary containing key and value pairs
-    #   that need to match the container. Note that the value of "*" can be used
-    #   as a wild card. This will ensure that any container that has the
-    #   specified key in the meta data is found.
-    #   \param container_type \type{class} An optional type of container to
-    #   filter on.
+    #   \param criteria A dictionary containing key and value pairs that need to
+    #   match the container. Note that the value of "*" can be used as a wild
+    #   card. This will ensure that any container that has the specified key in
+    #   the meta data is found.
+    #   \param container_type An optional type of container to filter on.
     #   \return The first container that matches the filter criteria or None if not found.
     @UM.FlameProfiler.profile
-    def findContainer(self, criteria = None, container_type = None, **kwargs) -> Optional[ContainerInterface]:
+    def findContainer(self, criteria: Dict[str, Any] = None, container_type: type = None, **kwargs: Any) -> Optional[ContainerInterface]:
         if not criteria and kwargs:
             criteria = kwargs
         elif criteria is None:
@@ -554,21 +569,22 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     ##  Add a container to the top of the stack.
     #
     #   \param container The container to add to the stack.
-    def addContainer(self, container):
+    def addContainer(self, container: ContainerInterface) -> None:
         self.insertContainer(0, container)
 
     ##  Insert a container into the stack.
     #
-    #   \param index \type{int} The index of to insert the container at.
+    #   \param index The index of to insert the container at.
     #          A negative index counts from the bottom
     #   \param container The container to add to the stack.
-    def insertContainer(self, index, container):
+    def insertContainer(self, index: int, container: ContainerInterface) -> None:
         if container is self:
             raise Exception("Unable to add stack to itself.")
 
         container.propertyChanged.connect(self._collectPropertyChanges)
         self._containers.insert(index, container)
         self.containersChanged.emit(container)
+        self._dirty = True
 
     ##  Replace a container in the stack.
     #
@@ -578,7 +594,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     #
     #   \exception IndexError Raised when the specified index is out of bounds.
     #   \exception Exception when trying to replace container ContainerStack.
-    def replaceContainer(self, index: int, container: ContainerInterface, postpone_emit=False):
+    def replaceContainer(self, index: int, container: ContainerInterface, postpone_emit: bool = False) -> None:
         if index < 0:
             raise IndexError
         if container is self:
@@ -587,6 +603,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
         self._containers[index].propertyChanged.disconnect(self._collectPropertyChanges)
         container.propertyChanged.connect(self._collectPropertyChanges)
         self._containers[index] = container
+        self._dirty = True
         if postpone_emit:
             # send it using sendPostponedEmits
             self._postponed_emits.append((self.containersChanged, container))
@@ -598,11 +615,12 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     #   \param index \type{int} The index of the container to remove.
     #
     #   \exception IndexError Raised when the specified index is out of bounds.
-    def removeContainer(self, index: int = 0):
+    def removeContainer(self, index: int = 0) -> None:
         if index < 0:
             raise IndexError
         try:
             container = self._containers[index]
+            self._dirty = True
             container.propertyChanged.disconnect(self._collectPropertyChanges)
             del self._containers[index]
             self.containersChanged.emit(container)
@@ -623,7 +641,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     #   \param stack \type{ContainerStack} The next stack to set. Can be None.
     #   Raises Exception when trying to set itself as next stack (to prevent infinite loops)
     #   \sa getNextStack
-    def setNextStack(self, stack: "ContainerStack", connect_signals: bool = True):
+    def setNextStack(self, stack: "ContainerStack", connect_signals: bool = True) -> None:
         if self is stack:
             raise Exception("Next stack can not be itself")
         if self._next_stack == stack:
@@ -640,7 +658,7 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     ##  Send postponed emits
     #   These emits are collected from the option postpone_emit.
     #   Note: the option can be implemented for all functions modifying the stack.
-    def sendPostponedEmits(self):
+    def sendPostponedEmits(self) -> None:
         while self._postponed_emits:
             signal, signal_arg = self._postponed_emits.pop(0)
             signal.emit(signal_arg)
@@ -657,12 +675,12 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
                 # Setting is not validated. This can happen if there is only a setting definition.
                 # We do need to validate it, because a setting defintions value can be set by a function, which could
                 # be an invalid setting.
-                definition = self.getSettingDefinition(key)
+                definition = cast(SettingDefinition, self.getSettingDefinition(key))
                 validator_type = SettingDefinition.getValidatorForType(definition.type)
                 if validator_type:
                     validator = validator_type(key)
                     validation_state = validator(self)
-            if validation_state in (ValidatorState.Exception, ValidatorState.MaximumError, ValidatorState.MinimumError):
+            if validation_state in (ValidatorState.Exception, ValidatorState.MaximumError, ValidatorState.MinimumError, ValidatorState.Invalid):
                 return True
         return False
 
@@ -676,12 +694,12 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
                 # Setting is not validated. This can happen if there is only a setting definition.
                 # We do need to validate it, because a setting defintions value can be set by a function, which could
                 # be an invalid setting.
-                definition = self.getSettingDefinition(key)
+                definition = cast(SettingDefinition, self.getSettingDefinition(key))
                 validator_type = SettingDefinition.getValidatorForType(definition.type)
                 if validator_type:
                     validator = validator_type(key)
                     validation_state = validator(self)
-            if validation_state in (ValidatorState.Exception, ValidatorState.MaximumError, ValidatorState.MinimumError):
+            if validation_state in (ValidatorState.Exception, ValidatorState.MaximumError, ValidatorState.MinimumError, ValidatorState.Invalid):
                 error_keys.append(key)
         return error_keys
 
@@ -691,18 +709,19 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
     # loop can run. This prevents us from sending the same change signal multiple times.
     # In addition, it allows us to emit a single signal that reports all properties that
     # have changed.
-    def _collectPropertyChanges(self, key: str, property_name: str):
+    def _collectPropertyChanges(self, key: str, property_name: str) -> None:
         if key not in self._property_changes:
             self._property_changes[key] = set()
 
         self._property_changes[key].add(property_name)
 
         if not self._emit_property_changed_queued:
-            _containerRegistry.getApplication().callLater(self._emitCollectedPropertyChanges)
+            from UM.Application import Application
+            Application.getInstance().callLater(self._emitCollectedPropertyChanges)
             self._emit_property_changed_queued = True
 
     # Perform the emission of the change signals that were collected in a previous step.
-    def _emitCollectedPropertyChanges(self):
+    def _emitCollectedPropertyChanges(self) -> None:
         for key, property_names in self._property_changes.items():
             self.propertiesChanged.emit(key, property_names)
 
@@ -712,7 +731,12 @@ class ContainerStack(QObject, ContainerInterface, PluginObject):
         self._property_changes = {}
         self._emit_property_changed_queued = False
 
-_containerRegistry = None   # type:  ContainerRegistryInterface
+    def __str__(self) -> str:
+        return "%s(%s)" % (type(self).__name__, self.getId())
+
+
+_containerRegistry = ContainerRegistryInterface()  # type: ContainerRegistryInterface
+
 
 def setContainerRegistry(registry: ContainerRegistryInterface) -> None:
     global _containerRegistry
