@@ -1,14 +1,16 @@
-# Copyright (c) 2017 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 import configparser
 import io
 import copy
-from typing import Any, Dict, List, Optional
+from typing import Any, cast, Dict, List, Optional, Set, Tuple
 
 from PyQt5.QtCore import QObject, pyqtProperty, pyqtSignal
+from PyQt5.QtQml import QQmlEngine #To take ownership of this class ourselves.
 
 from UM.Settings.Interfaces import DefinitionContainerInterface
+from UM.Settings.PropertyEvaluationContext import PropertyEvaluationContext #For typing.
 from UM.Signal import Signal, signalemitter
 from UM.PluginObject import PluginObject
 from UM.Logger import Logger
@@ -17,15 +19,19 @@ from UM.MimeTypeDatabase import MimeTypeDatabase, MimeType
 from UM.Settings.Interfaces import ContainerInterface, ContainerRegistryInterface
 from UM.Settings.SettingInstance import SettingInstance
 
+
 class InvalidInstanceError(Exception):
     pass
 
+
 class IncorrectInstanceVersionError(Exception):
     pass
 
+
 class DefinitionNotFoundError(Exception):
     pass
 
+
 MimeTypeDatabase.addMimeType(
     MimeType(
         name = "application/x-uranium-instancecontainer",
@@ -40,13 +46,14 @@ MimeTypeDatabase.addMimeType(
 #
 @signalemitter
 class InstanceContainer(QObject, ContainerInterface, PluginObject):
-    Version = 3
+    Version = 4
 
     ##  Constructor
     #
     #   \param container_id A unique, machine readable/writable ID for this container.
-    def __init__(self, container_id: str, parent = None, *args, **kwargs):
-        super().__init__(parent = parent, *args, **kwargs)
+    def __init__(self, container_id: str, parent: QObject = None, *args: Any, **kwargs: Any) -> None:
+        super().__init__()
+        QQmlEngine.setObjectOwnership(self, QQmlEngine.CppOwnership)
 
         self._metadata = {
             "id": container_id,
@@ -55,35 +62,37 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
             "container_type": InstanceContainer
         }                               # type: Dict[str, Any]
         self._instances = {}            # type: Dict[str, SettingInstance]
-        self._read_only = False
-        self._dirty = False
-        self._path = ""
-        self._postponed_emits = []
+        self._read_only = False  # type: bool
+        self._dirty = False  # type: bool
+        self._path = ""  # type: str
+        self._postponed_emits = []  # type: List[Tuple[Signal, Tuple[str, str]]]
+        self._definition = None  # type: Optional[DefinitionContainerInterface]
 
-        self._cached_values = None
+        self._cached_values = None  # type: Optional[Dict[str, Any]]
 
-    def __hash__(self):
+    def __hash__(self) -> int:
         # We need to re-implement the hash, because we defined the __eq__ operator.
         # According to some, returning the ID is technically not right, as objects with the same value should return
         # the same hash. The way we use it, it is acceptable for objects with the same value to return a different hash.
         return id(self)
 
-    def __deepcopy__(self, memo):
+    def __deepcopy__(self, memo: Dict[int, object]) -> "InstanceContainer":
         new_container = self.__class__(self.getId())
-        new_container._metadata = copy.deepcopy(self._metadata, memo)
-        new_container._instances = copy.deepcopy(self._instances, memo)
+        new_container._metadata = cast(Dict[str, Any], copy.deepcopy(self._metadata, memo))
+        new_container._instances = cast(Dict[str, SettingInstance], copy.deepcopy(self._instances, memo))
         for instance in new_container._instances.values(): #Set the back-links of the new instances correctly to the copied container.
             instance._container = new_container
             instance.propertyChanged.connect(new_container.propertyChanged)
         new_container._read_only = self._read_only
         new_container._dirty = self._dirty
-        new_container._path = copy.deepcopy(self._path, memo)
-        new_container._cached_values = copy.deepcopy(self._cached_values, memo)
+        new_container._path = cast(str, copy.deepcopy(self._path, memo))
+        new_container._cached_values = cast(Optional[Dict[str, Any]], copy.deepcopy(self._cached_values, memo))
         return new_container
 
-    def __eq__(self, other):
+    def __eq__(self, other: object) -> bool:
         if type(self) != type(other):
             return False  # Type mismatch
+        other = cast(InstanceContainer, other)
 
         self._instantiateCachedValues()
         other._instantiateCachedValues()
@@ -109,19 +118,19 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
                 return False  # Other has an instance that this object does not have.
         return True
 
-    def __ne__(self, other):
+    def __ne__(self, other: object) -> bool:
         return not (self == other)
 
     ##  For pickle support
-    def __getnewargs__(self):
+    def __getnewargs__(self) -> Tuple[str]:
         return (self.getId(),)
 
     ##  For pickle support
-    def __getstate__(self):
+    def __getstate__(self) -> Dict[str, Any]:
         return self.__dict__
 
     ##  For pickle support
-    def __setstate__(self, state):
+    def __setstate__(self, state: Dict[str, Any]) -> None:
         self.__dict__.update(state)
 
     ##  \copydoc ContainerInterface::getId
@@ -132,7 +141,7 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
 
     id = pyqtProperty(str, fget = getId, constant = True)
 
-    def setCachedValues(self, cached_values):
+    def setCachedValues(self, cached_values: Dict[str, Any]) -> None:
         if not self._instances:
             self._cached_values = cached_values
         else:
@@ -145,13 +154,13 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
     ##  \copydoc ContainerInterface::getPath.
     #
     #   Reimplemented from ContainerInterface
-    def getPath(self):
+    def getPath(self) -> str:
         return self._path
 
     ##  \copydoc ContainerInterface::setPath
     #
     #   Reimplemented from ContainerInterface
-    def setPath(self, path):
+    def setPath(self, path: str) -> None:
         self._path = path
 
     ##  \copydoc ContainerInterface::getName
@@ -160,7 +169,7 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
     def getName(self) -> str:
         return self._metadata["name"]
 
-    def setName(self, name):
+    def setName(self, name: str) -> None:
         if name != self.getName():
             self._metadata["name"] = name
             self._dirty = True
@@ -186,13 +195,16 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
         return _containerRegistry.isReadOnly(self.getId())
     readOnly = pyqtProperty(bool, fget = getReadOnly)
 
+    def getNumInstances(self) -> int:
+        return len(self._instances)
+
     ##  \copydoc ContainerInterface::getMetaData
     #
     #   Reimplemented from ContainerInterface
-    def getMetaData(self):
+    def getMetaData(self) -> Dict[str, Any]:
         return self._metadata
 
-    def setMetaData(self, metadata):
+    def setMetaData(self, metadata: Dict[str, Any]) -> None:
         if metadata == self._metadata:
             return #No need to do anything or even emit the signal.
 
@@ -202,7 +214,8 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
             "id": self.getId(),
             "name": self.getName(),
             "definition": self.getMetaData().get("definition"),
-            "version": self.getMetaData().get("version", 0)
+            "version": self.getMetaData().get("version", 0),
+            "container_type": InstanceContainer
         }
         self._metadata.update(metadata)
         self._dirty = True
@@ -214,42 +227,26 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
     ##  \copydoc ContainerInterface::getMetaDataEntry
     #
     #   Reimplemented from ContainerInterface
-    def getMetaDataEntry(self, entry, default = None):
+    def getMetaDataEntry(self, entry: str, default = None) -> Any:
         return self._metadata.get(entry, default)
 
-    ##  Add a new entry to the metadata of this container.
-    #
-    #   \param key \type{str} The key of the new entry.
-    #   \param value The value of the new entry.
-    #
-    #   \note This does nothing if the key already exists.
-    def addMetaDataEntry(self, key, value):
-        if key not in self._metadata:
-            self._metadata[key] = value
-            self._dirty = True
-            self.metaDataChanged.emit(self)
-        else:
-            Logger.log("w", "Meta data with key %s was already added.", key)
-
     ##  Set a metadata entry to a certain value.
     #
     #   \param key The key of the metadata entry to set.
     #   \param value The new value of the metadata.
     #
     #   \note This does nothing if the key is not already added to the metadata.
-    def setMetaDataEntry(self, key, value):
-        if key in self._metadata:
+    def setMetaDataEntry(self, key: str, value: Any) -> None:
+        if key not in self._metadata or self._metadata[key] != value:
             self._metadata[key] = value
             self._dirty = True
             self.metaDataChanged.emit(self)
-        else:
-            Logger.log("w", "Meta data with key %s was not found. Unable to change.", key)
 
     ##  Check if this container is dirty, that is, if it changed from deserialization.
-    def isDirty(self):
+    def isDirty(self) -> bool:
         return self._dirty
 
-    def setDirty(self, dirty):
+    def setDirty(self, dirty: bool) -> None:
         if self._read_only:
             Logger.log("w", "Tried to set dirty on read-only object.")
         else:
@@ -258,7 +255,7 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
     ##  \copydoc ContainerInterface::getProperty
     #
     #   Reimplemented from ContainerInterface
-    def getProperty(self, key, property_name, context = None):
+    def getProperty(self, key: str, property_name: str, context: PropertyEvaluationContext = None) -> Any:
         self._instantiateCachedValues()
         if key in self._instances:
             try:
@@ -271,7 +268,7 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
     ##  \copydoc ContainerInterface::hasProperty
     #
     #   Reimplemented from ContainerInterface.
-    def hasProperty(self, key, property_name):
+    def hasProperty(self, key: str, property_name: str) -> bool:
         # --- Kinda a hack:
         # When we check if a property exists, it is not necessary to flush the cache because we simply want to know
         # whether it is there. Flushing the cache can cause propertyChanged signals being emitted, and, as a result,
@@ -281,14 +278,14 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
         # **WITHOUT** applying the cached values. This way there won't be any property changed signals when we are
         # just checking if a property exists.
         #
-        self._instantiateMissingSettingInstancesInCache()
         if self._cached_values and key in self._cached_values and property_name == "value":
             return True
+        self._instantiateMissingSettingInstancesInCache()
         return key in self._instances and hasattr(self._instances[key], property_name)
 
     ##  Creates SettingInstances that are missing in this InstanceContainer from the cache if any.
     #   This function will **ONLY instantiate SettingInstances. The cached values will not be applied.**
-    def _instantiateMissingSettingInstancesInCache(self):
+    def _instantiateMissingSettingInstancesInCache(self) -> None:
         if not self._cached_values:
             return
 
@@ -313,26 +310,28 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
     #   If no instance has been created for the specified key, a new one will be created and inserted
     #   into this instance.
     #
-    #   \param key \type{string} The key of the setting to set a property of.
-    #   \param property_name \type{string} The name of the property to set.
+    #   \param key The key of the setting to set a property of.
+    #   \param property_name  The name of the property to set.
     #   \param property_value The new value of the property.
     #   \param container The container to use for retrieving values when changing the property triggers property updates. Defaults to None, which means use the current container.
     #   \param set_from_cache Flag to indicate that the property was set from cache. This triggers the behavior that the read_only and setDirty are ignored.
     #
     #   \note If no definition container is set for this container, new instances cannot be created and this method will do nothing.
-    def setProperty(self, key, property_name, property_value, container = None, set_from_cache = False):
+    def setProperty(self, key: str, property_name: str, property_value: Any, container: ContainerInterface = None, set_from_cache: bool = False) -> None:
         if self._read_only and not set_from_cache:
             Logger.log(
                 "w",
                 "Tried to setProperty [%s] with value [%s] with key [%s] on read-only object [%s]" % (
                     property_name, property_value, key, self.id))
             return
-        if key not in self._instances:
-            if not self.getDefinition():
-                Logger.log("w", "Tried to set value of setting %s that has no instance in container %s and the container has no definition", key, self.getName())
-                return
 
-            setting_definition = self.getDefinition().findDefinitions(key = key)
+        if key not in self._instances:
+            try:
+                definition = self.getDefinition()
+            except DefinitionNotFoundError:
+                Logger.log("w", "Tried to set value of setting when the container has no definition")
+                return
+            setting_definition = definition.findDefinitions(key = key)
             if not setting_definition:
                 Logger.log("w", "Tried to set value of setting %s that has no instance in container %s or its definition %s", key, self.getName(), self.getDefinition().getName())
                 return
@@ -341,7 +340,8 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
             instance.propertyChanged.connect(self.propertyChanged)
             self._instances[instance.definition.key] = instance
 
-        self._instances[key].setProperty(property_name, property_value, container)
+        # Do not emit any signal if the value is set from cache
+        self._instances[key].setProperty(property_name, property_value, container, emit_signals = not set_from_cache)
 
         if not set_from_cache:
             self.setDirty(True)
@@ -349,7 +349,7 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
     propertyChanged = Signal()
 
     ##  Remove all instances from this container.
-    def clear(self):
+    def clear(self) -> None:
         self._instantiateCachedValues()
         all_keys = self._instances.copy()
         for key in all_keys:
@@ -358,20 +358,20 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
 
     ##  Get all the keys of the instances of this container
     #   \returns list of keys
-    def getAllKeys(self):
+    def getAllKeys(self) -> Set[str]:
         keys = set(key for key in self._instances)
         if self._cached_values:
             # If we only want the keys and the actual values are still cached, just get the keys from the cache.
             keys.update(self._cached_values.keys())
-        return list(keys)
+        return keys
 
     ##  Create a new InstanceContainer with the same contents as this container
     #
-    #   \param new_id \type{str} The new ID of the container
-    #   \param new_name \type{str} The new name of the container. Defaults to None to indicate the name should not change.
+    #   \param new_id The new ID of the container
+    #   \param new_name The new name of the container. Defaults to None to indicate the name should not change.
     #
     #   \return A new InstanceContainer with the same contents as this container.
-    def duplicate(self, new_id: str, new_name: str = None):
+    def duplicate(self, new_id: str, new_name: str = None) -> "InstanceContainer":
         self._instantiateCachedValues()
         new_container = self.__class__(new_id)
         new_metadata = copy.deepcopy(self._metadata)
@@ -404,7 +404,9 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
         self._instantiateCachedValues()
         parser = configparser.ConfigParser(interpolation = None)
 
-        if not self.getDefinition():
+        try:
+            self.getDefinition()
+        except DefinitionNotFoundError:
             Logger.log("w", "Tried to serialize an instance container without definition, this is not supported")
             return ""
 
@@ -438,12 +440,12 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
     def _readAndValidateSerialized(cls, serialized: str) -> configparser.ConfigParser:
         # Disable comments in the ini files, so text values can start with a ;
         # without being removed as a comment
-        parser = configparser.ConfigParser(interpolation=None, comment_prefixes = ())
+        parser = configparser.ConfigParser(interpolation = None, comment_prefixes = ())
         parser.read_string(serialized)
 
         has_general = "general" in parser
-        has_version = "version" in parser["general"]
-        has_definition = "definition" in parser["general"]
+        has_version = has_general and "version" in parser["general"]
+        has_definition = has_general and "definition" in parser["general"]
 
         if not has_general or not has_version or not has_definition:
             exception_string = "Missing the required"
@@ -462,6 +464,8 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
         try:
             parser = cls._readAndValidateSerialized(serialized)
             configuration_type = parser["metadata"].get("type")
+        except InvalidInstanceError as iie:
+            raise iie
         except Exception as e:
             Logger.log("d", "Could not get configuration type: %s", e)
         return configuration_type
@@ -469,6 +473,9 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
     @classmethod
     def getVersionFromSerialized(cls, serialized: str) -> Optional[int]:
         configuration_type = cls.getConfigurationTypeFromSerialized(serialized)
+        if configuration_type is None:
+            Logger.log("w", "Could not determine configuration type.")
+            return None
         # get version
         version = None
         try:
@@ -490,7 +497,7 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
 
         try:
             parser_version = int(parser["general"]["version"])
-        except ValueError: #Version number is not integer.
+        except ValueError:  # Version number is not integer.
             raise IncorrectInstanceVersionError("Reported version {0} is not an integer.".format(parser["general"]["version"]))
         if parser_version != self.Version:
             raise IncorrectInstanceVersionError("Reported version {0} but expected version {1}".format(str(parser_version), self.Version))
@@ -529,7 +536,7 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
     #   instead.
     @classmethod
     def deserializeMetadata(cls, serialized: str, container_id: str) -> List[Dict[str, Any]]:
-        serialized = cls._updateSerialized(serialized) #Update to most recent version.
+        serialized = cls._updateSerialized(serialized)  # Update to most recent version.
         parser = configparser.ConfigParser(interpolation = None)
         parser.read_string(serialized)
 
@@ -541,8 +548,8 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
             metadata["name"] = parser["general"]["name"]
             metadata["version"] = parser["general"]["version"]
             metadata["definition"] = parser["general"]["definition"]
-        except KeyError: #One of the keys or the General section itself is missing.
-            return []
+        except KeyError as e: #One of the keys or the General section itself is missing.
+            raise InvalidInstanceError("Missing required fields: {error_msg}".format(error_msg = str(e)))
 
         if "metadata" in parser:
             metadata = {**metadata, **parser["metadata"]}
@@ -550,19 +557,19 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
         return [metadata]
 
     ##  Instance containers are lazy loaded. This function ensures that it happened.
-    def _instantiateCachedValues(self):
+    def _instantiateCachedValues(self) -> None:
         if not self._cached_values:
             return
-
+        definition = self.getDefinition()
         for key, value in self._cached_values.items():
-            self.setProperty(key, "value", value, self.getDefinition(), set_from_cache=True)
+            self.setProperty(key, "value", value, definition, set_from_cache=True)
 
         self._cached_values = None
 
     ##  Find instances matching certain criteria.
     #
-    #   \param kwargs \type{dict} A dictionary of keyword arguments with key-value pairs that should match properties of the instances.
-    def findInstances(self, **kwargs) -> List[SettingInstance]:
+    #   \param kwargs A dictionary of keyword arguments with key-value pairs that should match properties of the instances.
+    def findInstances(self, **kwargs: Any) -> List[SettingInstance]:
         self._instantiateCachedValues()
         result = []
         for setting_key, instance in self._instances.items():
@@ -596,7 +603,7 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
 
     ##  Remove an instance from this container.
     #   /param postpone_emit postpone emit until calling sendPostponedEmits
-    def removeInstance(self, key: str, postpone_emit: bool=False) -> None:
+    def removeInstance(self, key: str, postpone_emit: bool = False) -> None:
         self._instantiateCachedValues()
         if key not in self._instances:
             return
@@ -625,7 +632,7 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
         instance.updateRelations(self)
 
     ##  Update all instances from this container.
-    def update(self):
+    def update(self) -> None:
         self._instantiateCachedValues()
         for key, instance in self._instances.items():
             instance.propertyChanged.emit(key, "value")
@@ -638,41 +645,47 @@ class InstanceContainer(QObject, ContainerInterface, PluginObject):
 
     ##  Get the DefinitionContainer used for new instance creation.
     def getDefinition(self) -> DefinitionContainerInterface:
-        definitions = _containerRegistry.findDefinitionContainers(id = self._metadata["definition"])
-        if not definitions:
-            raise DefinitionNotFoundError("Could not find definition {0} required for instance {1}".format(self._metadata["definition"], self.getId()))
-        return definitions[0]
+        if self._definition is None:
+            definitions = _containerRegistry.findDefinitionContainers(id = self._metadata.get("definition", ""))
+            if not definitions:
+                raise DefinitionNotFoundError("Could not find definition {0} required for instance {1}".format(self._metadata.get("definition", ""), self.getId()))
+            self._definition = definitions[0]
+        return self._definition
 
     ##  Set the DefinitionContainer to use for new instance creation.
     #
     #   Since SettingInstance needs a SettingDefinition to work properly, we need some
     #   way of figuring out what SettingDefinition to use when creating a new SettingInstance.
-    def setDefinition(self, definition_id: str):
+    def setDefinition(self, definition_id: str) -> None:
         self._metadata["definition"] = definition_id
+        self._definition = None
 
-    def __lt__(self, other):
+    def __lt__(self, other: object) -> bool:
+        if type(other) != type(self):
+            return True
+        other = cast(InstanceContainer, other)
         own_weight = int(self.getMetaDataEntry("weight", 0))
         other_weight = int(other.getMetaDataEntry("weight", 0))
-
         if own_weight and other_weight:
             return own_weight < other_weight
 
         return self.getName() < other.getName()
 
     ##  Simple string representation for debugging.
-    def __str__(self):
+    def __str__(self) -> str:
         return "<InstanceContainer '{container_id}' ('{name}')>".format(container_id = self.getId(), name = self.getName())
 
-    ##  Send postponed emits
+    ##  Send the postponed emits
     #   These emits are collected from the option postpone_emit.
     #   Note: the option can be implemented for all functions modifying the container.
-    def sendPostponedEmits(self):
+    def sendPostponedEmits(self) -> None:
         while self._postponed_emits:
             signal, signal_arg = self._postponed_emits.pop(0)
             signal.emit(*signal_arg)
 
 
-_containerRegistry = None   # type:  ContainerRegistryInterface
+_containerRegistry = ContainerRegistryInterface()  # type:  ContainerRegistryInterface
+
 
 def setContainerRegistry(registry: ContainerRegistryInterface) -> None:
     global _containerRegistry
