@@ -1,28 +1,34 @@
-# Copyright (c) 2017 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
-import os #To get the IDs from file names.
-from typing import List, Dict, Any, Optional
-import urllib.parse #To get the IDs from file names.
+from typing import Any, Dict, List, Optional, Set, TYPE_CHECKING
 
 import UM.Decorators
 from UM.Logger import Logger
-from UM.MimeTypeDatabase import MimeTypeDatabase
 from UM.Signal import Signal
 from UM.Settings.PropertyEvaluationContext import PropertyEvaluationContext
 
+if TYPE_CHECKING:
+    from UM.Application import Application
+    from UM.Settings.InstanceContainer import InstanceContainer
+    from UM.Settings.SettingDefinition import SettingDefinition
+
 
 ##  Shared interface between setting container types
 #
 @UM.Decorators.interface
 class ContainerInterface:
+
+    def __init__(self, *args, **kwargs):
+        pass
+
     ##  Get the ID of the container.
     #
     #   The ID should be unique, machine readable and machine writable. It is
     #   intended to be used for example when referencing the container in
     #   configuration files or when writing a file to disk.
     #
-    #   \return \type{string} The unique ID of this container.
+    #   \return The unique ID of this container.
     def getId(self) -> str:
         pass
 
@@ -31,7 +37,7 @@ class ContainerInterface:
     #   This should return a human-readable name for the container, that can be
     #   used in the interface.
     #
-    #   \return \type{string} The name of this container.
+    #   \return The name of this container.
     def getName(self) -> str:
         pass
 
@@ -40,13 +46,13 @@ class ContainerInterface:
     #   This returns a dictionary containing all the metadata for this container.
     #   How this metadata is used depends on the application.
     #
-    #   \return \type{dict} The metadata for this container.
+    #   \return The metadata for this container.
     def getMetaData(self) -> Dict[str, Any]:
         pass
 
     ##  Get the value of a single metadata entry.
     #
-    #   \param entry \type{string} The key of the metadata to retrieve.
+    #   \param entry The key of the metadata to retrieve.
     #   \param default The default value to return if the entry cannot be found.
     #
     #   \return The value of the metadata corresponding to `name`, or `default`
@@ -55,10 +61,8 @@ class ContainerInterface:
         pass
 
     ##  Get the value of a property of the container item.
-    #
-    #   \param key \type{string} The key of the item to retrieve a property from.
-    #   \param name \type{string} The name of the property to retrieve.
-    #
+    #   \param key The key of the item to retrieve a property from.
+    #   \param property_name The name of the property to retrieve.
     #   \return The specified property value of the container item corresponding to key, or None if not found.
     def getProperty(self, key: str, property_name: str, context: Optional[PropertyEvaluationContext] = None) -> Any:
         pass
@@ -73,6 +77,11 @@ class ContainerInterface:
     def hasProperty(self, key: str, property_name: str) -> bool:
         pass
 
+    ##  Get all the setting keys known to this container.
+    #   \return Set of keys.
+    def getAllKeys(self) -> Set[str]:
+        pass
+
     ##  Serialize this container to a string.
     #
     #   The serialized representation of the container can be used to write the
@@ -81,10 +90,23 @@ class ContainerInterface:
     #   \param ignored_metadata_keys A set of keys that should be ignored when
     #   it serializes the metadata.
     #
-    #   \return \type{string} A string representation of this container.
+    #   \return A string representation of this container.
     def serialize(self, ignored_metadata_keys: Optional[set] = None) -> str:
         pass
 
+    ##  Change a property of a container item.
+    #   \param key The key of the item to change the property of.
+    #   \param property_name The name of the property to change.
+    #   \param property_value The new value of the property.
+    #   \param container The container to use for retrieving values when
+    #   changing the property triggers property updates. Defaults to None, which
+    #   means use the current container.
+    #   \param set_from_cache Flag to indicate that the property was set from
+    #   cache. This triggers the behavior that the read_only and setDirty are
+    #   ignored.
+    def setProperty(self, key: str, property_name: str, property_value: Any, container: "ContainerInterface" = None, set_from_cache: bool = False) -> None:
+        pass
+
     ##  Deserialize the container from a string representation.
     #
     #   This should replace the contents of this container with those in the serialized
@@ -142,17 +164,39 @@ class ContainerInterface:
     def setPath(self, path: str) -> None:
         pass
 
+    def isDirty(self) -> bool:
+        pass
+
     propertyChanged = None   # type: Signal
 
     metaDataChanged = None  # type: Signal
 
 
+@UM.Decorators.interface
 class DefinitionContainerInterface(ContainerInterface):
-    pass
+    def findDefinitions(self, **kwargs: Any) -> List["SettingDefinition"]:
+        raise NotImplementedError()
+
+    def setProperty(self, key: str, property_name: str, property_value: Any, container: "ContainerInterface" = None, set_from_cache: bool = False) -> None:
+        raise TypeError("Can't change properties in definition containers.")
 
 
 ##  Shared interface between setting container types
 #
 @UM.Decorators.interface
 class ContainerRegistryInterface:
-    def findDefinitionContainers(self, **kwargs: Any) -> List[DefinitionContainerInterface]: pass
+    def findContainers(self, *, ignore_case: bool = False, **kwargs: Any) -> List[ContainerInterface]:
+        raise NotImplementedError()
+
+    def findDefinitionContainers(self, **kwargs: Any) -> List[DefinitionContainerInterface]:
+        raise NotImplementedError()
+
+    @classmethod
+    def getApplication(cls) -> "Application":
+        raise NotImplementedError()
+
+    def getEmptyInstanceContainer(self) -> "InstanceContainer":
+        raise NotImplementedError()
+
+    def isReadOnly(self, container_id: str) -> bool:
+        raise NotImplementedError()
\ No newline at end of file
