@@ -10,13 +10,14 @@ from UM.Settings.ContainerRegistry import ContainerRegistry
 from UM.Settings.SettingDefinition import SettingDefinition
 from UM.Settings.DefinitionContainer import DefinitionContainer
 
+
 ##  This class provides the value and change notifications for the properties of a single setting
 #
 #   This class provides the property values through QObject dynamic properties so that they
 #   are available from QML.
 class ContainerPropertyProvider(QObject):
-    def __init__(self, parent = None, *args, **kwargs):
-        super().__init__(parent = parent, *args, **kwargs)
+    def __init__(self, parent = None):
+        super().__init__(parent = parent)
 
         self._container_id = ""
         self._container = None
@@ -27,7 +28,7 @@ class ContainerPropertyProvider(QObject):
     ##  Set the containerId property.
     def setContainerId(self, container_id):
         if container_id == self._container_id:
-            return #No change.
+            return # No change.
 
         self._container_id = container_id
 
@@ -119,7 +120,6 @@ class ContainerPropertyProvider(QObject):
 
         if property_name not in self._watched_properties:
             return
-
         value = self._getPropertyValue(property_name)
 
         if self._property_values.get(property_name, None) != value:
