@@ -1,4 +1,4 @@
-# Copyright (c) 2017 Ultimaker B.V.
+# Copyright (c) 2019 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 from UM.Qt.ListModel import ListModel
@@ -7,6 +7,7 @@ from PyQt5.QtCore import pyqtProperty, Qt, pyqtSignal
 
 from UM.Settings.ContainerRegistry import ContainerRegistry
 from UM.Settings.DefinitionContainer import DefinitionContainer
+from typing import Dict
 
 
 ##  Model that holds definition containers. By setting the filter property the definitions held by this model can be
@@ -27,7 +28,11 @@ class DefinitionContainersModel(ListModel):
         ContainerRegistry.getInstance().containerRemoved.connect(self._onContainerChanged)
 
         self._section_property = ""
-        self._preferred_section_value = ""
+
+        #Preference for which sections should be shown on top. Weights for each section.
+        #Sections with the lowest value are shown on top. Sections not on this
+        #list will get a value of 0.
+        self._preferred_sections = {} #type: Dict[str, int]
 
         self._filter_dict = {}
         self._update()
@@ -45,7 +50,7 @@ class DefinitionContainersModel(ListModel):
         definition_containers.sort(key = self._sortKey)
 
         for metadata in definition_containers:
-            metadata = metadata.copy()
+            metadata = dict(metadata) # For fully loaded definitions, the metadata is an OrderedDict which does not pass to QML correctly
 
             items.append({
                 "name": metadata["name"],
@@ -66,16 +71,17 @@ class DefinitionContainersModel(ListModel):
     def sectionProperty(self):
         return self._section_property
 
-    def setPreferredSectionValue(self, value):
-        if self._preferred_section_value != value:
-            self._preferred_section_value = value
-            self.preferredSectionValueChanged.emit()
+    def setPreferredSections(self, weights: Dict[str, int]):
+        if self._preferred_sections != weights:
+            self._preferred_sections = weights
+            self.preferredSectionsChanged.emit()
             self._update()
 
-    preferredSectionValueChanged = pyqtSignal()
-    @pyqtProperty(str, fset = setPreferredSectionValue, notify = preferredSectionValueChanged)
-    def preferredSectionValue(self):
-        return self._preferred_section_value
+    preferredSectionsChanged = pyqtSignal()
+
+    @pyqtProperty("QVariantMap", fset = setPreferredSections, notify = preferredSectionsChanged)
+    def preferredSections(self):
+        return self._preferred_sections
 
     ##  Set the filter of this model based on a string.
     #   \param filter_dict Dictionary to do the filtering by.
@@ -93,12 +99,12 @@ class DefinitionContainersModel(ListModel):
 
         if self._section_property:
             section_value = item.get(self._section_property, "")
-            if self._preferred_section_value:
-                result.append(section_value != self._preferred_section_value)
-            result.append(section_value)
+            section_weight = self._preferred_sections.get(section_value, 0)
+            result.append(section_weight)
+            result.append(section_value.lower())
 
-        result.append(int(item.get("weight", 0)))
-        result.append(item["name"])
+        result.append(int(item.get("weight", 0))) #Weight within a section.
+        result.append(item["name"].lower())
 
         return result
 
