@@ -1,7 +1,7 @@
-# Copyright (c) 2016 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
-from UM.Preferences import Preferences
+from UM.Application import Application
 from UM.OutputDevice.OutputDevicePlugin import OutputDevicePlugin
 from .LocalFileOutputDevice import LocalFileOutputDevice
 
@@ -14,8 +14,8 @@ class LocalFileOutputDevicePlugin(OutputDevicePlugin):
     def __init__(self):
         super().__init__()
 
-        Preferences.getInstance().addPreference("local_file/last_used_type", "")
-        Preferences.getInstance().addPreference("local_file/dialog_save_path", "")
+        Application.getInstance().getPreferences().addPreference("local_file/last_used_type", "")
+        Application.getInstance().getPreferences().addPreference("local_file/dialog_save_path", "")
 
     def start(self):
         self.getOutputDeviceManager().addOutputDevice(LocalFileOutputDevice())
