@@ -1,6 +1,7 @@
-# Copyright (c) 2015 Ultimaker B.V.
+# Copyright (c) 2019 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
-from UM.Benchmark import Benchmark
+from typing import List, Tuple, TYPE_CHECKING, Optional
+
 from UM.Tool import Tool
 from UM.Event import Event, MouseEvent, KeyEvent
 from UM.Scene.ToolHandle import ToolHandle
@@ -13,17 +14,21 @@ from UM.Math.Matrix import Matrix
 from UM.Operations.ScaleOperation import ScaleOperation
 from UM.Operations.GroupedOperation import GroupedOperation
 from UM.Operations.SetTransformOperation import SetTransformOperation
-from UM.Operations.ScaleToBoundsOperation import ScaleToBoundsOperation
 
 from PyQt5.QtCore import Qt
-from . import ScaleToolHandle
+try:
+    from . import ScaleToolHandle
+except (ImportError, SystemError):
+    import ScaleToolHandle  # type: ignore  # This fixes the tests not being able to import.
 
 import scipy
+if TYPE_CHECKING:
+    from UM.Scene.SceneNode import SceneNode
 
 DIMENSION_TOLERANCE = 0.0001    # Tolerance value used for comparing dimensions from the UI.
 
-##  Provides the tool to scale meshes and groups
 
+##  Provides the tool to scale meshes and groups
 class ScaleTool(Tool):
     def __init__(self):
         super().__init__()
@@ -35,18 +40,17 @@ class ScaleTool(Tool):
 
         self._drag_length = 0
 
-        self._maximum_bounds = None
         self._move_up = True
 
-        self._shortcut_key = Qt.Key_A
+        self._shortcut_key = Qt.Key_S
 
         # We use the position of the scale handle when the operation starts.
         # This is done in order to prevent runaway reactions (drag changes of 100+)
         self._saved_handle_position = None  # for non uniform drag
         self._scale_sum = 0.0  # a memory for uniform drag with snap scaling
-        self._last_event = None  # for uniform drag
+        self._last_event = None  # type: Optional[Event] # for uniform drag
 
-        self._saved_node_positions = []
+        self._saved_node_positions = []  # type: List[Tuple[SceneNode, Vector]]
 
         self.setExposedProperties(
             "ScaleSnap",
@@ -66,29 +70,27 @@ class ScaleTool(Tool):
         super().event(event)
 
         if event.type == Event.ToolActivateEvent:
-            for node in Selection.getAllSelectedObjects():
+            for node in self._getSelectedObjectsWithoutSelectedAncestors():
                 node.boundingBoxChanged.connect(self.propertyChanged)
 
         if event.type == Event.ToolDeactivateEvent:
-            for node in Selection.getAllSelectedObjects():
+            for node in self._getSelectedObjectsWithoutSelectedAncestors():
                 node.boundingBoxChanged.disconnect(self.propertyChanged)
 
         # Handle modifier keys: Shift toggles snap, Control toggles uniform scaling
         if event.type == Event.KeyPressEvent:
             if event.key == KeyEvent.ShiftKey:
-                self._snap_scale = False
-                self.propertyChanged.emit()
+                self.setScaleSnap(not self._snap_scale)
+
             elif event.key == KeyEvent.ControlKey:
-                self._non_uniform_scale = True
-                self.propertyChanged.emit()
+                self.setNonUniformScale(not self._non_uniform_scale)
 
         if event.type == Event.KeyReleaseEvent:
             if event.key == KeyEvent.ShiftKey:
-                self._snap_scale = True
-                self.propertyChanged.emit()
+                self.setScaleSnap(not self._snap_scale)
+
             elif event.key == KeyEvent.ControlKey:
-                self._non_uniform_scale = False
-                self.propertyChanged.emit()
+                self.setNonUniformScale(not self._non_uniform_scale)
 
         if event.type == Event.MousePressEvent and self._controller.getToolsEnabled():
             # Initialise a scale operation
@@ -105,7 +107,7 @@ class ScaleTool(Tool):
 
             # Save the current positions of the node, as we want to scale arround their current centres
             self._saved_node_positions = []
-            for node in Selection.getAllSelectedObjects():
+            for node in self._getSelectedObjectsWithoutSelectedAncestors():
                 self._saved_node_positions.append((node, node.getPosition()))
 
             self._scale_sum = 0.0
@@ -121,6 +123,7 @@ class ScaleTool(Tool):
                 self.setDragPlane(Plane(Vector(0, 1, 0), self._saved_handle_position.y))
 
             self.setDragStart(event.x, event.y)
+            return True
 
         if event.type == Event.MouseMoveEvent:
             # Perform a scale operation
@@ -129,6 +132,13 @@ class ScaleTool(Tool):
 
             drag_position = self.getDragPosition(event.x, event.y)
             if drag_position:
+                if self.getLockedAxis() == ToolHandle.XAxis:
+                    drag_position = drag_position.set(y = 0, z = 0)
+                elif self.getLockedAxis() == ToolHandle.YAxis:
+                    drag_position = drag_position.set(x = 0, z = 0)
+                elif self.getLockedAxis() == ToolHandle.ZAxis:
+                    drag_position = drag_position.set(x = 0, y = 0)
+
                 drag_length = (drag_position - self._saved_handle_position).length()
                 if self._drag_length > 0:
                     drag_change = (drag_length - self._drag_length) / 100 * self._scale_speed
@@ -166,10 +176,14 @@ class ScaleTool(Tool):
                             scale_change = scale_change.set(x=scale_factor, y=scale_factor, z=scale_factor)
 
                         # Scale around the saved centers of all selected nodes
-                        op = GroupedOperation()
-                        for node, position in self._saved_node_positions:
-                            op.addOperation(ScaleOperation(node, scale_change, relative_scale = True, scale_around_point = position))
-                        op.push()
+                        if len(self._saved_node_positions) > 1:
+                            op = GroupedOperation()
+                            for node, position in self._saved_node_positions:
+                                op.addOperation(ScaleOperation(node, scale_change, relative_scale = True, scale_around_point = position))
+                            op.push()
+                        else:
+                            for node, position in self._saved_node_positions:
+                                ScaleOperation(node, scale_change, relative_scale = True, scale_around_point = position).push()
                         self._drag_length = (self._saved_handle_position - drag_position).length()
                 else:
                     self.operationStarted.emit(self)
@@ -181,7 +195,7 @@ class ScaleTool(Tool):
             # Finish a scale operation
             if self.getDragPlane():
                 self.setDragPlane(None)
-                self.setLockedAxis(None)
+                self.setLockedAxis(ToolHandle.NoAxis)
                 self._drag_length = 0
                 self.operationStopped.emit(self)
                 return True
@@ -190,11 +204,6 @@ class ScaleTool(Tool):
     def resetScale(self):
         Selection.applyOperation(SetTransformOperation, None, None, Vector(1.0, 1.0, 1.0), Vector(0, 0, 0))
 
-    ##  Initialise and start a ScaleToBoundsOperation on the selected objects
-    def scaleToMax(self):
-        if hasattr(self.getController().getScene(), "_maximum_bounds"):
-            Selection.applyOperation(ScaleToBoundsOperation, self.getController().getScene()._maximum_bounds)
-
     ##  Get non-uniform scaling flag
     #
     #   \return scale type(boolean)
@@ -295,12 +304,7 @@ class ScaleTool(Tool):
                 else:
                     scale_vector = Vector(scale_factor, scale_factor, scale_factor)
 
-                op = GroupedOperation()
-                for node in Selection.getAllSelectedObjects():
-                    op.addOperation(
-                        ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
-                op.push()
-
+                self._scaleSelectedNodes(scale_vector)
     ##  Set the height of the selected object(s) by scaling the first selected object to a certain height
     #
     #   \param height type(float) height in mm
@@ -316,11 +320,7 @@ class ScaleTool(Tool):
                 else:
                     scale_vector = Vector(scale_factor, scale_factor, scale_factor)
 
-                op = GroupedOperation()
-                for node in Selection.getAllSelectedObjects():
-                    op.addOperation(
-                        ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
-                op.push()
+                self._scaleSelectedNodes(scale_vector)
 
     ##  Set the depth of the selected object(s) by scaling the first selected object to a certain depth
     #
@@ -337,11 +337,7 @@ class ScaleTool(Tool):
                 else:
                     scale_vector = Vector(scale_factor, scale_factor, scale_factor)
 
-                op = GroupedOperation()
-                for node in Selection.getAllSelectedObjects():
-                    op.addOperation(
-                        ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
-                op.push()
+                self._scaleSelectedNodes(scale_vector)
 
     ##  Set the x-scale of the selected object(s) by scaling the first selected object to a certain factor
     #
@@ -357,11 +353,7 @@ class ScaleTool(Tool):
                 else:
                     scale_vector = Vector(scale_factor, scale_factor, scale_factor)
 
-                op = GroupedOperation()
-                for node in Selection.getAllSelectedObjects():
-                    op.addOperation(
-                        ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
-                op.push()
+                self._scaleSelectedNodes(scale_vector)
 
     ##  Set the y-scale of the selected object(s) by scaling the first selected object to a certain factor
     #
@@ -377,11 +369,7 @@ class ScaleTool(Tool):
                 else:
                     scale_vector = Vector(scale_factor, scale_factor, scale_factor)
 
-                op = GroupedOperation()
-                for node in Selection.getAllSelectedObjects():
-                    op.addOperation(
-                        ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
-                op.push()
+                self._scaleSelectedNodes(scale_vector)
 
     ##  Set the z-scale of the selected object(s) by scaling the first selected object to a certain factor
     #
@@ -397,11 +385,18 @@ class ScaleTool(Tool):
                 else:
                     scale_vector = Vector(scale_factor, scale_factor, scale_factor)
 
-                op = GroupedOperation()
-                for node in Selection.getAllSelectedObjects():
-                    op.addOperation(
-                        ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
-                op.push()
+                self._scaleSelectedNodes(scale_vector)
+
+    def _scaleSelectedNodes(self, scale_vector: Vector) -> None:
+        selected_nodes = self._getSelectedObjectsWithoutSelectedAncestors()
+        if len(selected_nodes) > 1:
+            op = GroupedOperation()
+            for node in selected_nodes:
+                op.addOperation(ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()))
+            op.push()
+        else:
+            for node in selected_nodes:
+                ScaleOperation(node, scale_vector, scale_around_point=node.getWorldPosition()).push()
 
     ##  Convenience function that gives the scale of an object in the coordinate space of the world.
     #   The function might return wrong value if the grouped models are rotated
