@@ -1,4 +1,4 @@
-# Copyright (c) 2017 Ultimaker B.V.
+# Copyright (c) 2018 Ultimaker B.V.
 # Uranium is released under the terms of the LGPLv3 or higher.
 
 # The purpose of this class is to create fixtures or methods that can be shared
@@ -8,6 +8,8 @@ import os.path
 import pytest
 import unittest.mock #For mocking the container provider priority.
 
+from UM.PluginRegistry import PluginRegistry
+from UM.VersionUpgradeManager import VersionUpgradeManager
 from UM.Resources import Resources
 from UM.MimeTypeDatabase import MimeType, MimeTypeDatabase
 from UM.Settings.ContainerProvider import ContainerProvider #To provide the fixtures for container providers.
@@ -23,7 +25,7 @@ import UM.Settings.InstanceContainer
 #
 #   \return A brand new container registry.
 @pytest.fixture
-def container_registry(application, test_containers_provider):
+def container_registry(application, test_containers_provider, plugin_registry: PluginRegistry):
     MimeTypeDatabase.addMimeType(
         MimeType(
             name = "application/x-uranium-definitioncontainer",
@@ -48,10 +50,8 @@ def container_registry(application, test_containers_provider):
         )
     )
 
-    ContainerRegistry.setApplication(application)
-
     ContainerRegistry._ContainerRegistry__instance = None # Reset the private instance variable every time
-    registry = ContainerRegistry.getInstance()
+    registry = ContainerRegistry(application)
 
     #We need to mock the "priority" plug-in metadata field, but preferably without mocking an entire plug-in.
     with unittest.mock.patch("UM.PluginRegistry.PluginRegistry.getMetaData", unittest.mock.MagicMock(return_value = {"container_provider": {}})):
@@ -76,20 +76,20 @@ def container_provider():
 ##  Container provider which provides the containers that are in the setting
 #   test directory.
 @pytest.fixture
-def test_containers_provider(container_provider: ContainerProvider) -> ContainerProvider:
+def test_containers_provider(container_provider: ContainerProvider, upgrade_manager: VersionUpgradeManager) -> ContainerProvider:
     my_folder = os.path.dirname(os.path.abspath(__file__))
 
     definition_ids = {"basic_definition", "children", "functions", "inherits", "metadata_definition", "multiple_settings", "single_setting"}
     for definition_id in definition_ids:
         container = DefinitionContainer(definition_id)
-        container.deserialize(open(os.path.join(my_folder, "definitions", definition_id + ".def.json")).read())
+        container.deserialize(open(os.path.join(my_folder, "definitions", definition_id + ".def.json"), encoding = "utf-8").read())
         container_provider._containers[definition_id] = container
         container_provider.addMetadata(container.getMetaData())
 
     instance_ids = {"basic_instance", "metadata_instance", "setting_values"}
     for instance_id in instance_ids:
         container = UM.Settings.InstanceContainer.InstanceContainer(instance_id)
-        container.deserialize(open(os.path.join(my_folder, "instances", instance_id + ".inst.cfg")).read())
+        container.deserialize(open(os.path.join(my_folder, "instances", instance_id + ".inst.cfg"), encoding = "utf-8").read())
         container_provider._containers[instance_id] = container
         container_provider.addMetadata(container.getMetaData())
 
@@ -100,6 +100,12 @@ def test_containers_provider(container_provider: ContainerProvider) -> Container
 #   To add something to this provider, add it to its `_containers` and its
 #   `_metadata` fields.
 class TestContainerProvider(ContainerProvider):
+
+    def __init__(self) -> None:
+        super().__init__()
+        self._plugin_id = "TestContainerProvider"
+        self._version = "0.1.0"
+
     def getAllIds(self, *args, **kwargs):
         return self._containers.keys()
 
@@ -110,4 +116,7 @@ class TestContainerProvider(ContainerProvider):
         return self._containers[container_id]
 
     def loadMetadata(self, container_id, *args, **kwargs):
-        return self._metadata[container_id]
\ No newline at end of file
+        return self._metadata[container_id]
+
+    def saveContainer(self, *args, **kwargs):
+        return None
\ No newline at end of file
